(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
    typeof define === 'function' && define.amd ? define(factory) :
    (global = global || self, global.DataGridExt = factory());
}(this, (function () { 'use strict';

    function noop() { }
    function run(fn) {
        return fn();
    }
    function blank_object() {
        return Object.create(null);
    }
    function run_all(fns) {
        fns.forEach(run);
    }
    function is_function(thing) {
        return typeof thing === 'function';
    }
    function safe_not_equal(a, b) {
        return a != a ? b == b : a !== b || ((a && typeof a === 'object') || typeof a === 'function');
    }
    function action_destroyer(action_result) {
        return action_result && is_function(action_result.destroy) ? action_result.destroy : noop;
    }

    function append(target, node) {
        target.appendChild(node);
    }
    function insert(target, node, anchor) {
        target.insertBefore(node, anchor || null);
    }
    function detach(node) {
        node.parentNode.removeChild(node);
    }
    function destroy_each(iterations, detaching) {
        for (let i = 0; i < iterations.length; i += 1) {
            if (iterations[i])
                iterations[i].d(detaching);
        }
    }
    function element(name) {
        return document.createElement(name);
    }
    function text(data) {
        return document.createTextNode(data);
    }
    function space() {
        return text(' ');
    }
    function empty() {
        return text('');
    }
    function listen(node, event, handler, options) {
        node.addEventListener(event, handler, options);
        return () => node.removeEventListener(event, handler, options);
    }
    function attr(node, attribute, value) {
        if (value == null)
            node.removeAttribute(attribute);
        else if (node.getAttribute(attribute) !== value)
            node.setAttribute(attribute, value);
    }
    function children(element) {
        return Array.from(element.childNodes);
    }
    function set_data(text, data) {
        data = '' + data;
        if (text.data !== data)
            text.data = data;
    }
    function set_style(node, key, value, important) {
        node.style.setProperty(key, value, important ? 'important' : '');
    }
    function add_resize_listener(element, fn) {
        if (getComputedStyle(element).position === 'static') {
            element.style.position = 'relative';
        }
        const object = document.createElement('object');
        object.setAttribute('style', 'display: block; position: absolute; top: 0; left: 0; height: 100%; width: 100%; overflow: hidden; pointer-events: none; z-index: -1;');
        object.setAttribute('aria-hidden', 'true');
        object.type = 'text/html';
        object.tabIndex = -1;
        let win;
        object.onload = () => {
            win = object.contentDocument.defaultView;
            win.addEventListener('resize', fn);
        };
        if (/Trident/.test(navigator.userAgent)) {
            element.appendChild(object);
            object.data = 'about:blank';
        }
        else {
            object.data = 'about:blank';
            element.appendChild(object);
        }
        return {
            cancel: () => {
                win && win.removeEventListener && win.removeEventListener('resize', fn);
                element.removeChild(object);
            }
        };
    }
    function custom_event(type, detail) {
        const e = document.createEvent('CustomEvent');
        e.initCustomEvent(type, false, false, detail);
        return e;
    }

    let current_component;
    function set_current_component(component) {
        current_component = component;
    }
    function get_current_component() {
        if (!current_component)
            throw new Error(`Function called outside component initialization`);
        return current_component;
    }
    function onMount(fn) {
        get_current_component().$$.on_mount.push(fn);
    }
    function createEventDispatcher() {
        const component = get_current_component();
        return (type, detail) => {
            const callbacks = component.$$.callbacks[type];
            if (callbacks) {
                // TODO are there situations where events could be dispatched
                // in a server (non-DOM) environment?
                const event = custom_event(type, detail);
                callbacks.slice().forEach(fn => {
                    fn.call(component, event);
                });
            }
        };
    }

    const dirty_components = [];
    const binding_callbacks = [];
    const render_callbacks = [];
    const flush_callbacks = [];
    const resolved_promise = Promise.resolve();
    let update_scheduled = false;
    function schedule_update() {
        if (!update_scheduled) {
            update_scheduled = true;
            resolved_promise.then(flush);
        }
    }
    function add_render_callback(fn) {
        render_callbacks.push(fn);
    }
    let flushing = false;
    const seen_callbacks = new Set();
    function flush() {
        if (flushing)
            return;
        flushing = true;
        do {
            // first, call beforeUpdate functions
            // and update components
            for (let i = 0; i < dirty_components.length; i += 1) {
                const component = dirty_components[i];
                set_current_component(component);
                update(component.$$);
            }
            dirty_components.length = 0;
            while (binding_callbacks.length)
                binding_callbacks.pop()();
            // then, once components are updated, call
            // afterUpdate functions. This may cause
            // subsequent updates...
            for (let i = 0; i < render_callbacks.length; i += 1) {
                const callback = render_callbacks[i];
                if (!seen_callbacks.has(callback)) {
                    // ...so guard against infinite loops
                    seen_callbacks.add(callback);
                    callback();
                }
            }
            render_callbacks.length = 0;
        } while (dirty_components.length);
        while (flush_callbacks.length) {
            flush_callbacks.pop()();
        }
        update_scheduled = false;
        flushing = false;
        seen_callbacks.clear();
    }
    function update($$) {
        if ($$.fragment !== null) {
            $$.update();
            run_all($$.before_update);
            const dirty = $$.dirty;
            $$.dirty = [-1];
            $$.fragment && $$.fragment.p($$.ctx, dirty);
            $$.after_update.forEach(add_render_callback);
        }
    }
    const outroing = new Set();
    let outros;
    function group_outros() {
        outros = {
            r: 0,
            c: [],
            p: outros // parent group
        };
    }
    function check_outros() {
        if (!outros.r) {
            run_all(outros.c);
        }
        outros = outros.p;
    }
    function transition_in(block, local) {
        if (block && block.i) {
            outroing.delete(block);
            block.i(local);
        }
    }
    function transition_out(block, local, detach, callback) {
        if (block && block.o) {
            if (outroing.has(block))
                return;
            outroing.add(block);
            outros.c.push(() => {
                outroing.delete(block);
                if (callback) {
                    if (detach)
                        block.d(1);
                    callback();
                }
            });
            block.o(local);
        }
    }

    const globals = (typeof window !== 'undefined' ? window : global);
    function outro_and_destroy_block(block, lookup) {
        transition_out(block, 1, 1, () => {
            lookup.delete(block.key);
        });
    }
    function update_keyed_each(old_blocks, dirty, get_key, dynamic, ctx, list, lookup, node, destroy, create_each_block, next, get_context) {
        let o = old_blocks.length;
        let n = list.length;
        let i = o;
        const old_indexes = {};
        while (i--)
            old_indexes[old_blocks[i].key] = i;
        const new_blocks = [];
        const new_lookup = new Map();
        const deltas = new Map();
        i = n;
        while (i--) {
            const child_ctx = get_context(ctx, list, i);
            const key = get_key(child_ctx);
            let block = lookup.get(key);
            if (!block) {
                block = create_each_block(key, child_ctx);
                block.c();
            }
            else if (dynamic) {
                block.p(child_ctx, dirty);
            }
            new_lookup.set(key, new_blocks[i] = block);
            if (key in old_indexes)
                deltas.set(key, Math.abs(i - old_indexes[key]));
        }
        const will_move = new Set();
        const did_move = new Set();
        function insert(block) {
            transition_in(block, 1);
            block.m(node, next, lookup.has(block.key));
            lookup.set(block.key, block);
            next = block.first;
            n--;
        }
        while (o && n) {
            const new_block = new_blocks[n - 1];
            const old_block = old_blocks[o - 1];
            const new_key = new_block.key;
            const old_key = old_block.key;
            if (new_block === old_block) {
                // do nothing
                next = new_block.first;
                o--;
                n--;
            }
            else if (!new_lookup.has(old_key)) {
                // remove old block
                destroy(old_block, lookup);
                o--;
            }
            else if (!lookup.has(new_key) || will_move.has(new_key)) {
                insert(new_block);
            }
            else if (did_move.has(old_key)) {
                o--;
            }
            else if (deltas.get(new_key) > deltas.get(old_key)) {
                did_move.add(new_key);
                insert(new_block);
            }
            else {
                will_move.add(old_key);
                o--;
            }
        }
        while (o--) {
            const old_block = old_blocks[o];
            if (!new_lookup.has(old_block.key))
                destroy(old_block, lookup);
        }
        while (n)
            insert(new_blocks[n - 1]);
        return new_blocks;
    }
    function create_component(block) {
        block && block.c();
    }
    function mount_component(component, target, anchor) {
        const { fragment, on_mount, on_destroy, after_update } = component.$$;
        fragment && fragment.m(target, anchor);
        // onMount happens before the initial afterUpdate
        add_render_callback(() => {
            const new_on_destroy = on_mount.map(run).filter(is_function);
            if (on_destroy) {
                on_destroy.push(...new_on_destroy);
            }
            else {
                // Edge case - component was destroyed immediately,
                // most likely as a result of a binding initialising
                run_all(new_on_destroy);
            }
            component.$$.on_mount = [];
        });
        after_update.forEach(add_render_callback);
    }
    function destroy_component(component, detaching) {
        const $$ = component.$$;
        if ($$.fragment !== null) {
            run_all($$.on_destroy);
            $$.fragment && $$.fragment.d(detaching);
            // TODO null out other refs, including component.$$ (but need to
            // preserve final state?)
            $$.on_destroy = $$.fragment = null;
            $$.ctx = [];
        }
    }
    function make_dirty(component, i) {
        if (component.$$.dirty[0] === -1) {
            dirty_components.push(component);
            schedule_update();
            component.$$.dirty.fill(0);
        }
        component.$$.dirty[(i / 31) | 0] |= (1 << (i % 31));
    }
    function init(component, options, instance, create_fragment, not_equal, props, dirty = [-1]) {
        const parent_component = current_component;
        set_current_component(component);
        const prop_values = options.props || {};
        const $$ = component.$$ = {
            fragment: null,
            ctx: null,
            // state
            props,
            update: noop,
            not_equal,
            bound: blank_object(),
            // lifecycle
            on_mount: [],
            on_destroy: [],
            before_update: [],
            after_update: [],
            context: new Map(parent_component ? parent_component.$$.context : []),
            // everything else
            callbacks: blank_object(),
            dirty
        };
        let ready = false;
        $$.ctx = instance
            ? instance(component, prop_values, (i, ret, ...rest) => {
                const value = rest.length ? rest[0] : ret;
                if ($$.ctx && not_equal($$.ctx[i], $$.ctx[i] = value)) {
                    if ($$.bound[i])
                        $$.bound[i](value);
                    if (ready)
                        make_dirty(component, i);
                }
                return ret;
            })
            : [];
        $$.update();
        ready = true;
        run_all($$.before_update);
        // `false` as a special case of no DOM component
        $$.fragment = create_fragment ? create_fragment($$.ctx) : false;
        if (options.target) {
            if (options.hydrate) {
                const nodes = children(options.target);
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                $$.fragment && $$.fragment.l(nodes);
                nodes.forEach(detach);
            }
            else {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                $$.fragment && $$.fragment.c();
            }
            if (options.intro)
                transition_in(component.$$.fragment);
            mount_component(component, options.target, options.anchor);
            flush();
        }
        set_current_component(parent_component);
    }
    class SvelteComponent {
        $destroy() {
            destroy_component(this, 1);
            this.$destroy = noop;
        }
        $on(type, callback) {
            const callbacks = (this.$$.callbacks[type] || (this.$$.callbacks[type] = []));
            callbacks.push(callback);
            return () => {
                const index = callbacks.indexOf(callback);
                if (index !== -1)
                    callbacks.splice(index, 1);
            };
        }
        $set() {
            // overridden by instance, if it has props
        }
    }

    (function(root, factory) { // eslint-disable-line no-extra-semi
      var deepDiff = factory(root);
      // eslint-disable-next-line no-undef
      if (typeof define === 'function' && define.amd) {
          // AMD
          define('DeepDiff', function() { // eslint-disable-line no-undef
              return deepDiff;
          });
      } else if (typeof exports === 'object' || typeof navigator === 'object' && navigator.product.match(/ReactNative/i)) {
          // Node.js or ReactNative
          module.exports = deepDiff;
      } else {
          // Browser globals
          var _deepdiff = root.DeepDiff;
          deepDiff.noConflict = function() {
              if (root.DeepDiff === deepDiff) {
                  root.DeepDiff = _deepdiff;
              }
              return deepDiff;
          };
          root.DeepDiff = deepDiff;
      }
    }(undefined, function(root) {
      var validKinds = ['N', 'E', 'A', 'D'];

      // nodejs compatible on server side and in the browser.
      function inherits(ctor, superCtor) {
        ctor.super_ = superCtor;
        ctor.prototype = Object.create(superCtor.prototype, {
          constructor: {
            value: ctor,
            enumerable: false,
            writable: true,
            configurable: true
          }
        });
      }

      function Diff(kind, path) {
        Object.defineProperty(this, 'kind', {
          value: kind,
          enumerable: true
        });
        if (path && path.length) {
          Object.defineProperty(this, 'path', {
            value: path,
            enumerable: true
          });
        }
      }

      function DiffEdit(path, origin, value) {
        DiffEdit.super_.call(this, 'E', path);
        Object.defineProperty(this, 'lhs', {
          value: origin,
          enumerable: true
        });
        Object.defineProperty(this, 'rhs', {
          value: value,
          enumerable: true
        });
      }
      inherits(DiffEdit, Diff);

      function DiffNew(path, value) {
        DiffNew.super_.call(this, 'N', path);
        Object.defineProperty(this, 'rhs', {
          value: value,
          enumerable: true
        });
      }
      inherits(DiffNew, Diff);

      function DiffDeleted(path, value) {
        DiffDeleted.super_.call(this, 'D', path);
        Object.defineProperty(this, 'lhs', {
          value: value,
          enumerable: true
        });
      }
      inherits(DiffDeleted, Diff);

      function DiffArray(path, index, item) {
        DiffArray.super_.call(this, 'A', path);
        Object.defineProperty(this, 'index', {
          value: index,
          enumerable: true
        });
        Object.defineProperty(this, 'item', {
          value: item,
          enumerable: true
        });
      }
      inherits(DiffArray, Diff);

      function arrayRemove(arr, from, to) {
        var rest = arr.slice((to || from) + 1 || arr.length);
        arr.length = from < 0 ? arr.length + from : from;
        arr.push.apply(arr, rest);
        return arr;
      }

      function realTypeOf(subject) {
        var type = typeof subject;
        if (type !== 'object') {
          return type;
        }

        if (subject === Math) {
          return 'math';
        } else if (subject === null) {
          return 'null';
        } else if (Array.isArray(subject)) {
          return 'array';
        } else if (Object.prototype.toString.call(subject) === '[object Date]') {
          return 'date';
        } else if (typeof subject.toString === 'function' && /^\/.*\//.test(subject.toString())) {
          return 'regexp';
        }
        return 'object';
      }

      // http://werxltd.com/wp/2010/05/13/javascript-implementation-of-javas-string-hashcode-method/
      function hashThisString(string) {
        var hash = 0;
        if (string.length === 0) { return hash; }
        for (var i = 0; i < string.length; i++) {
          var char = string.charCodeAt(i);
          hash = ((hash << 5) - hash) + char;
          hash = hash & hash; // Convert to 32bit integer
        }
        return hash;
      }

      // Gets a hash of the given object in an array order-independent fashion
      // also object key order independent (easier since they can be alphabetized)
      function getOrderIndependentHash(object) {
        var accum = 0;
        var type = realTypeOf(object);

        if (type === 'array') {
          object.forEach(function (item) {
            // Addition is commutative so this is order indep
            accum += getOrderIndependentHash(item);
          });

          var arrayString = '[type: array, hash: ' + accum + ']';
          return accum + hashThisString(arrayString);
        }

        if (type === 'object') {
          for (var key in object) {
            if (object.hasOwnProperty(key)) {
              var keyValueString = '[ type: object, key: ' + key + ', value hash: ' + getOrderIndependentHash(object[key]) + ']';
              accum += hashThisString(keyValueString);
            }
          }

          return accum;
        }

        // Non object, non array...should be good?
        var stringToHash = '[ type: ' + type + ' ; value: ' + object + ']';
        return accum + hashThisString(stringToHash);
      }

      function deepDiff(lhs, rhs, changes, prefilter, path, key, stack, orderIndependent) {
        changes = changes || [];
        path = path || [];
        stack = stack || [];
        var currentPath = path.slice(0);
        if (typeof key !== 'undefined' && key !== null) {
          if (prefilter) {
            if (typeof (prefilter) === 'function' && prefilter(currentPath, key)) {
              return;
            } else if (typeof (prefilter) === 'object') {
              if (prefilter.prefilter && prefilter.prefilter(currentPath, key)) {
                return;
              }
              if (prefilter.normalize) {
                var alt = prefilter.normalize(currentPath, key, lhs, rhs);
                if (alt) {
                  lhs = alt[0];
                  rhs = alt[1];
                }
              }
            }
          }
          currentPath.push(key);
        }

        // Use string comparison for regexes
        if (realTypeOf(lhs) === 'regexp' && realTypeOf(rhs) === 'regexp') {
          lhs = lhs.toString();
          rhs = rhs.toString();
        }

        var ltype = typeof lhs;
        var rtype = typeof rhs;
        var i, j, k, other;

        var ldefined = ltype !== 'undefined' ||
          (stack && (stack.length > 0) && stack[stack.length - 1].lhs &&
            Object.getOwnPropertyDescriptor(stack[stack.length - 1].lhs, key));
        var rdefined = rtype !== 'undefined' ||
          (stack && (stack.length > 0) && stack[stack.length - 1].rhs &&
            Object.getOwnPropertyDescriptor(stack[stack.length - 1].rhs, key));

        if (!ldefined && rdefined) {
          changes.push(new DiffNew(currentPath, rhs));
        } else if (!rdefined && ldefined) {
          changes.push(new DiffDeleted(currentPath, lhs));
        } else if (realTypeOf(lhs) !== realTypeOf(rhs)) {
          changes.push(new DiffEdit(currentPath, lhs, rhs));
        } else if (realTypeOf(lhs) === 'date' && (lhs - rhs) !== 0) {
          changes.push(new DiffEdit(currentPath, lhs, rhs));
        } else if (ltype === 'object' && lhs !== null && rhs !== null) {
          for (i = stack.length - 1; i > -1; --i) {
            if (stack[i].lhs === lhs) {
              other = true;
              break;
            }
          }
          if (!other) {
            stack.push({ lhs: lhs, rhs: rhs });
            if (Array.isArray(lhs)) {
              // If order doesn't matter, we need to sort our arrays
              if (orderIndependent) {
                lhs.sort(function (a, b) {
                  return getOrderIndependentHash(a) - getOrderIndependentHash(b);
                });

                rhs.sort(function (a, b) {
                  return getOrderIndependentHash(a) - getOrderIndependentHash(b);
                });
              }
              i = rhs.length - 1;
              j = lhs.length - 1;
              while (i > j) {
                changes.push(new DiffArray(currentPath, i, new DiffNew(undefined, rhs[i--])));
              }
              while (j > i) {
                changes.push(new DiffArray(currentPath, j, new DiffDeleted(undefined, lhs[j--])));
              }
              for (; i >= 0; --i) {
                deepDiff(lhs[i], rhs[i], changes, prefilter, currentPath, i, stack, orderIndependent);
              }
            } else {
              var akeys = Object.keys(lhs);
              var pkeys = Object.keys(rhs);
              for (i = 0; i < akeys.length; ++i) {
                k = akeys[i];
                other = pkeys.indexOf(k);
                if (other >= 0) {
                  deepDiff(lhs[k], rhs[k], changes, prefilter, currentPath, k, stack, orderIndependent);
                  pkeys[other] = null;
                } else {
                  deepDiff(lhs[k], undefined, changes, prefilter, currentPath, k, stack, orderIndependent);
                }
              }
              for (i = 0; i < pkeys.length; ++i) {
                k = pkeys[i];
                if (k) {
                  deepDiff(undefined, rhs[k], changes, prefilter, currentPath, k, stack, orderIndependent);
                }
              }
            }
            stack.length = stack.length - 1;
          } else if (lhs !== rhs) {
            // lhs is contains a cycle at this element and it differs from rhs
            changes.push(new DiffEdit(currentPath, lhs, rhs));
          }
        } else if (lhs !== rhs) {
          if (!(ltype === 'number' && isNaN(lhs) && isNaN(rhs))) {
            changes.push(new DiffEdit(currentPath, lhs, rhs));
          }
        }
      }

      function observableDiff(lhs, rhs, observer, prefilter, orderIndependent) {
        var changes = [];
        deepDiff(lhs, rhs, changes, prefilter, null, null, null, orderIndependent);
        if (observer) {
          for (var i = 0; i < changes.length; ++i) {
            observer(changes[i]);
          }
        }
        return changes;
      }

      function orderIndependentDeepDiff(lhs, rhs, changes, prefilter, path, key, stack) {
        return deepDiff(lhs, rhs, changes, prefilter, path, key, stack, true);
      }

      function accumulateDiff(lhs, rhs, prefilter, accum) {
        var observer = (accum) ?
          function (difference) {
            if (difference) {
              accum.push(difference);
            }
          } : undefined;
        var changes = observableDiff(lhs, rhs, observer, prefilter);
        return (accum) ? accum : (changes.length) ? changes : undefined;
      }

      function accumulateOrderIndependentDiff(lhs, rhs, prefilter, accum) {
        var observer = (accum) ?
          function (difference) {
            if (difference) {
              accum.push(difference);
            }
          } : undefined;
        var changes = observableDiff(lhs, rhs, observer, prefilter, true);
        return (accum) ? accum : (changes.length) ? changes : undefined;
      }

      function applyArrayChange(arr, index, change) {
        if (change.path && change.path.length) {
          var it = arr[index],
            i, u = change.path.length - 1;
          for (i = 0; i < u; i++) {
            it = it[change.path[i]];
          }
          switch (change.kind) {
            case 'A':
              applyArrayChange(it[change.path[i]], change.index, change.item);
              break;
            case 'D':
              delete it[change.path[i]];
              break;
            case 'E':
            case 'N':
              it[change.path[i]] = change.rhs;
              break;
          }
        } else {
          switch (change.kind) {
            case 'A':
              applyArrayChange(arr[index], change.index, change.item);
              break;
            case 'D':
              arr = arrayRemove(arr, index);
              break;
            case 'E':
            case 'N':
              arr[index] = change.rhs;
              break;
          }
        }
        return arr;
      }

      function applyChange(target, source, change) {
        if (typeof change === 'undefined' && source && ~validKinds.indexOf(source.kind)) {
          change = source;
        }
        if (target && change && change.kind) {
          var it = target,
            i = -1,
            last = change.path ? change.path.length - 1 : 0;
          while (++i < last) {
            if (typeof it[change.path[i]] === 'undefined') {
              it[change.path[i]] = (typeof change.path[i + 1] !== 'undefined' && typeof change.path[i + 1] === 'number') ? [] : {};
            }
            it = it[change.path[i]];
          }
          switch (change.kind) {
            case 'A':
              if (change.path && typeof it[change.path[i]] === 'undefined') {
                it[change.path[i]] = [];
              }
              applyArrayChange(change.path ? it[change.path[i]] : it, change.index, change.item);
              break;
            case 'D':
              delete it[change.path[i]];
              break;
            case 'E':
            case 'N':
              it[change.path[i]] = change.rhs;
              break;
          }
        }
      }

      function revertArrayChange(arr, index, change) {
        if (change.path && change.path.length) {
          // the structure of the object at the index has changed...
          var it = arr[index],
            i, u = change.path.length - 1;
          for (i = 0; i < u; i++) {
            it = it[change.path[i]];
          }
          switch (change.kind) {
            case 'A':
              revertArrayChange(it[change.path[i]], change.index, change.item);
              break;
            case 'D':
              it[change.path[i]] = change.lhs;
              break;
            case 'E':
              it[change.path[i]] = change.lhs;
              break;
            case 'N':
              delete it[change.path[i]];
              break;
          }
        } else {
          // the array item is different...
          switch (change.kind) {
            case 'A':
              revertArrayChange(arr[index], change.index, change.item);
              break;
            case 'D':
              arr[index] = change.lhs;
              break;
            case 'E':
              arr[index] = change.lhs;
              break;
            case 'N':
              arr = arrayRemove(arr, index);
              break;
          }
        }
        return arr;
      }

      function revertChange(target, source, change) {
        if (target && source && change && change.kind) {
          var it = target,
            i, u;
          u = change.path.length - 1;
          for (i = 0; i < u; i++) {
            if (typeof it[change.path[i]] === 'undefined') {
              it[change.path[i]] = {};
            }
            it = it[change.path[i]];
          }
          switch (change.kind) {
            case 'A':
              // Array was modified...
              // it will be an array...
              revertArrayChange(it[change.path[i]], change.index, change.item);
              break;
            case 'D':
              // Item was deleted...
              it[change.path[i]] = change.lhs;
              break;
            case 'E':
              // Item was edited...
              it[change.path[i]] = change.lhs;
              break;
            case 'N':
              // Item is new...
              delete it[change.path[i]];
              break;
          }
        }
      }

      function applyDiff(target, source, filter) {
        if (target && source) {
          var onChange = function (change) {
            if (!filter || filter(target, source, change)) {
              applyChange(target, source, change);
            }
          };
          observableDiff(target, source, onChange);
        }
      }

      Object.defineProperties(accumulateDiff, {

        diff: {
          value: accumulateDiff,
          enumerable: true
        },
        orderIndependentDiff: {
          value: accumulateOrderIndependentDiff,
          enumerable: true
        },
        observableDiff: {
          value: observableDiff,
          enumerable: true
        },
        orderIndependentObservableDiff: {
          value: orderIndependentDeepDiff,
          enumerable: true
        },
        orderIndepHash: {
          value: getOrderIndependentHash,
          enumerable: true
        },
        applyDiff: {
          value: applyDiff,
          enumerable: true
        },
        applyChange: {
          value: applyChange,
          enumerable: true
        },
        revertChange: {
          value: revertChange,
          enumerable: true
        },
        isConflict: {
          value: function () {
            return typeof $conflict !== 'undefined';
          },
          enumerable: true
        }
      });

      // hackish...
      accumulateDiff.DeepDiff = accumulateDiff;
      // ...but works with:
      // import DeepDiff from 'deep-diff'
      // import { DeepDiff } from 'deep-diff'
      // const DeepDiff = require('deep-diff');
      // const { DeepDiff } = require('deep-diff');

      if (root) {
        root.DeepDiff = accumulateDiff;
      }

      return accumulateDiff;
    }));

    var DeepDiff = /*#__PURE__*/Object.freeze({
        __proto__: null
    });

    const applyChange = undefined;
    const diff = undefined;
    /**
     * Edit history tracker for a javascript object using deep-diff to generate and apply patches
     */
    class EditHistory {
      /**
       * Instantiates an instance of EditHistory
       * @param {Object} obj The object or array to track
       */
      constructor(obj) {
        this.obj = JSON.parse(JSON.stringify(obj));

        // initialize arrays for forwards and backwards patches
        this.forward = [];
        this.backward = [];
      }

      /**
       * Clears all forward and backward patches
       */
      clear() {
        this.forward = [];
        this.backward = [];
      }

      /**
       * Records a change to an object
       * @param {Object} newObj The new object
       */
      recordChange(newObj) {
        const patch = {
          redo: diff(this.obj, newObj),
          undo: diff(newObj, this.obj)
        };

        if (!patch.redo || !patch.undo) {
          console.warn("Objects could not be diffed");
        } else {
          this.obj = JSON.parse(JSON.stringify(newObj));
          this.backward.push(patch);
        }
      }

      /**
       * Applies the most recent undo patch and returns the new object
       * @returns {Object} The tracked object
       */
      undo() {
        if (this.backward.length === 0) {
          return null;
        }

        // grab the most recent backwards patch
        const patch = this.backward.pop();

        // applyChange doesn't accept arrays, only its members
        patch.undo.forEach(x => applyChange(this.obj, x));

        // put the patch into the forward queue
        this.forward.push(patch);

        return JSON.parse(JSON.stringify(this.obj));
      }

      /**
       * Applies the most recent redo patch and returns the new object
       * @returns {Object} The tracked object
       */
      redo() {
        if (this.forward.length === 0) {
          return null;
        }

        // grab the most recent forwards patch
        const patch = this.forward.pop();

        // applyChange doesn't accept arrays, only its members
        patch.redo.forEach(x => applyChange(this.obj, x));

        // put the patch into the backward queue
        this.backward.push(patch);

        return JSON.parse(JSON.stringify(this.obj));
      }

      /**
       * Applies all the undo patches in the queue and returns the new object
       * @returns {Object} The tracked object
       */
      undoAll() {
        while (this.backward.length > 0) {
          this.undo();
        }

        return this.obj;
      }

      /**
       * Applies all the redo patches in the queue and returns the new object
       * @returns {Object} The tracked object
       */
      redoAll() {
        while (this.forward.length > 0) {
          this.redo();
        }

        return this.obj;
      }
    }

    /* src/index.svelte generated by Svelte v3.20.1 */

    const { document: document_1, window: window_1 } = globals;

    function add_css() {
    	var style = element("style");
    	style.id = "svelte-b58rik-style";
    	style.textContent = ".row-action-line.svelte-b58rik.svelte-b58rik{position:absolute;left:0;width:100%;height:4px;z-index:6;background:#aaa}.row-affix-marker.svelte-b58rik.svelte-b58rik{display:none;position:absolute;left:0;height:4px;z-index:6;background:white;border-top:1px solid #999;border-bottom:1px solid #999;cursor:ns-resize}.column-affix-marker.svelte-b58rik.svelte-b58rik{position:absolute;top:0;width:4px;z-index:5;background:white;border-left:1px solid #999;border-right:1px solid #999;cursor:ew-resize;transform:translateX(-50%)}.resizing.svelte-b58rik .svelte-b58rik{user-select:none}.resizing.svelte-b58rik .grid-inner.svelte-b58rik{overflow-y:hidden}.resizing.svelte-b58rik .grid-space.svelte-b58rik{pointer-events:all}.grid-cell.svelte-b58rik>.svelte-b58rik{height:100%}.cell-default.svelte-b58rik.svelte-b58rik{padding:0 5px;background:white;overflow:hidden;text-overflow:ellipsis}.data-grid-wrapper.svelte-b58rik.svelte-b58rik{position:relative;width:100%;height:100%}.column-action-line.svelte-b58rik.svelte-b58rik{position:absolute;top:0;bottom:17px;z-index:3;width:4px;background:#aaa;cursor:ew-resize}.grid-cell-size-capture.svelte-b58rik.svelte-b58rik{position:absolute;top:0;bottom:0;z-index:5;background:transparent;cursor:ew-resize;pointer-events:all}.grid-inner.svelte-b58rik.svelte-b58rik{overflow:auto}.grid-space.svelte-b58rik.svelte-b58rik{position:absolute;top:0;left:0;background:transparent;pointer-events:none;z-index:3}.grid-headers.svelte-b58rik.svelte-b58rik{position:absolute;overflow:hidden;max-width:100%;width:100%;top:0;left:0;border-bottom:2px solid black}.grid-headers.svelte-b58rik .grid-cell.svelte-b58rik{text-align:center;font-weight:bold;cursor:pointer}.grid-headers.svelte-b58rik .cell-default.svelte-b58rik:hover{background:#eee}.grid-header-row.svelte-b58rik.svelte-b58rik{position:absolute;overflow:hidden;top:0}.grid-row.svelte-b58rik.svelte-b58rik{position:absolute;overflow:hidden;white-space:nowrap;text-overflow:ellipsis}.grid-row.svelte-b58rik.svelte-b58rik:not(:last-child){border-bottom:1px solid #666}.grid-cell.svelte-b58rik.svelte-b58rik{position:absolute;top:0;text-overflow:ellipsis;overflow:hidden}.grid-cell.svelte-b58rik>.svelte-b58rik{width:100%}.grid-cell.svelte-b58rik.svelte-b58rik:not(:last-child){border-right:1px solid #666}";
    	append(document_1.head, style);
    }

    function get_each_context_1(ctx, list, i) {
    	const child_ctx = ctx.slice();
    	child_ctx[69] = list[i];
    	child_ctx[71] = i;
    	return child_ctx;
    }

    function get_each_context(ctx, list, i) {
    	const child_ctx = ctx.slice();
    	child_ctx[66] = list[i];
    	child_ctx[68] = i;
    	return child_ctx;
    }

    function get_each_context_2(ctx, list, i) {
    	const child_ctx = ctx.slice();
    	child_ctx[69] = list[i];
    	child_ctx[68] = i;
    	return child_ctx;
    }

    function get_each_context_3(ctx, list, i) {
    	const child_ctx = ctx.slice();
    	child_ctx[69] = list[i];
    	child_ctx[68] = i;
    	return child_ctx;
    }

    // (900:2) {#if __resizing || __columnDragging || __affixingColumn}
    function create_if_block_7(ctx) {
    	let div;

    	return {
    		c() {
    			div = element("div");
    			attr(div, "class", "column-action-line svelte-b58rik");
    			set_style(div, "left", /*__columnActionLineLeft*/ ctx[6] - 2 + "px");
    		},
    		m(target, anchor) {
    			insert(target, div, anchor);
    		},
    		p(ctx, dirty) {
    			if (dirty[0] & /*__columnActionLineLeft*/ 64) {
    				set_style(div, "left", /*__columnActionLineLeft*/ ctx[6] - 2 + "px");
    			}
    		},
    		d(detaching) {
    			if (detaching) detach(div);
    		}
    	};
    }

    // (905:2) {#if __affixingRow}
    function create_if_block_6(ctx) {
    	let div;

    	return {
    		c() {
    			div = element("div");
    			attr(div, "class", "row-action-line svelte-b58rik");
    			set_style(div, "top", /*__rowActionLineTop*/ ctx[16] - 2 + "px");
    		},
    		m(target, anchor) {
    			insert(target, div, anchor);
    		},
    		p(ctx, dirty) {
    			if (dirty[0] & /*__rowActionLineTop*/ 65536) {
    				set_style(div, "top", /*__rowActionLineTop*/ ctx[16] - 2 + "px");
    			}
    		},
    		d(detaching) {
    			if (detaching) detach(div);
    		}
    	};
    }

    // (929:10) {:else}
    function create_else_block_1(ctx) {
    	let div;
    	let t_value = (/*column*/ ctx[69].display || "") + "";
    	let t;

    	return {
    		c() {
    			div = element("div");
    			t = text(t_value);
    			attr(div, "class", "cell-default svelte-b58rik");
    		},
    		m(target, anchor) {
    			insert(target, div, anchor);
    			append(div, t);
    		},
    		p(ctx, dirty) {
    			if (dirty[0] & /*columns*/ 1 && t_value !== (t_value = (/*column*/ ctx[69].display || "") + "")) set_data(t, t_value);
    		},
    		i: noop,
    		o: noop,
    		d(detaching) {
    			if (detaching) detach(div);
    		}
    	};
    }

    // (927:10) {#if column.headerComponent}
    function create_if_block_5(ctx) {
    	let switch_instance_anchor;
    	let current;
    	var switch_value = /*column*/ ctx[69].headerComponent;

    	function switch_props(ctx) {
    		return { props: { column: /*column*/ ctx[69] } };
    	}

    	if (switch_value) {
    		var switch_instance = new switch_value(switch_props(ctx));
    	}

    	return {
    		c() {
    			if (switch_instance) create_component(switch_instance.$$.fragment);
    			switch_instance_anchor = empty();
    		},
    		m(target, anchor) {
    			if (switch_instance) {
    				mount_component(switch_instance, target, anchor);
    			}

    			insert(target, switch_instance_anchor, anchor);
    			current = true;
    		},
    		p(ctx, dirty) {
    			const switch_instance_changes = {};
    			if (dirty[0] & /*columns*/ 1) switch_instance_changes.column = /*column*/ ctx[69];

    			if (switch_value !== (switch_value = /*column*/ ctx[69].headerComponent)) {
    				if (switch_instance) {
    					group_outros();
    					const old_component = switch_instance;

    					transition_out(old_component.$$.fragment, 1, 0, () => {
    						destroy_component(old_component, 1);
    					});

    					check_outros();
    				}

    				if (switch_value) {
    					switch_instance = new switch_value(switch_props(ctx));
    					create_component(switch_instance.$$.fragment);
    					transition_in(switch_instance.$$.fragment, 1);
    					mount_component(switch_instance, switch_instance_anchor.parentNode, switch_instance_anchor);
    				} else {
    					switch_instance = null;
    				}
    			} else if (switch_value) {
    				switch_instance.$set(switch_instance_changes);
    			}
    		},
    		i(local) {
    			if (current) return;
    			if (switch_instance) transition_in(switch_instance.$$.fragment, local);
    			current = true;
    		},
    		o(local) {
    			if (switch_instance) transition_out(switch_instance.$$.fragment, local);
    			current = false;
    		},
    		d(detaching) {
    			if (detaching) detach(switch_instance_anchor);
    			if (switch_instance) destroy_component(switch_instance, detaching);
    		}
    	};
    }

    // (933:8) {#if allowResizeFromTableHeaders && !column.disallowResize}
    function create_if_block_4(ctx) {
    	let div;
    	let dispose;

    	function mousedown_handler_1(...args) {
    		return /*mousedown_handler_1*/ ctx[60](/*i*/ ctx[68], ...args);
    	}

    	return {
    		c() {
    			div = element("div");
    			attr(div, "class", "grid-cell-size-capture svelte-b58rik");

    			set_style(div, "left", getCellLeft({
    				i: /*i*/ ctx[68],
    				columnWidths: /*columnWidths*/ ctx[21],
    				__affixedColumnIndices: /*__affixedColumnIndices*/ ctx[1],
    				__scrollLeft: /*__scrollLeft*/ ctx[8]
    			}) + /*columnWidths*/ ctx[21][/*i*/ ctx[68]] - Math.floor(/*__columnHeaderResizeCaptureWidth*/ ctx[15] / 2) + "px");

    			set_style(div, "width", /*__columnHeaderResizeCaptureWidth*/ ctx[15] + "px");
    		},
    		m(target, anchor, remount) {
    			insert(target, div, anchor);
    			if (remount) dispose();
    			dispose = listen(div, "mousedown", mousedown_handler_1);
    		},
    		p(new_ctx, dirty) {
    			ctx = new_ctx;

    			if (dirty[0] & /*columns, columnWidths, __affixedColumnIndices, __scrollLeft, __columnHeaderResizeCaptureWidth*/ 2130179) {
    				set_style(div, "left", getCellLeft({
    					i: /*i*/ ctx[68],
    					columnWidths: /*columnWidths*/ ctx[21],
    					__affixedColumnIndices: /*__affixedColumnIndices*/ ctx[1],
    					__scrollLeft: /*__scrollLeft*/ ctx[8]
    				}) + /*columnWidths*/ ctx[21][/*i*/ ctx[68]] - Math.floor(/*__columnHeaderResizeCaptureWidth*/ ctx[15] / 2) + "px");
    			}

    			if (dirty[0] & /*__columnHeaderResizeCaptureWidth*/ 32768) {
    				set_style(div, "width", /*__columnHeaderResizeCaptureWidth*/ ctx[15] + "px");
    			}
    		},
    		d(detaching) {
    			if (detaching) detach(div);
    			dispose();
    		}
    	};
    }

    // (917:6) {#each columns as column, i (i)}
    function create_each_block_3(key_1, ctx) {
    	let div;
    	let current_block_type_index;
    	let if_block0;
    	let div_title_value;
    	let dragCopy_action;
    	let t;
    	let if_block1_anchor;
    	let current;
    	let dispose;
    	const if_block_creators = [create_if_block_5, create_else_block_1];
    	const if_blocks = [];

    	function select_block_type(ctx, dirty) {
    		if (/*column*/ ctx[69].headerComponent) return 0;
    		return 1;
    	}

    	current_block_type_index = select_block_type(ctx);
    	if_block0 = if_blocks[current_block_type_index] = if_block_creators[current_block_type_index](ctx);

    	function mousedown_handler(...args) {
    		return /*mousedown_handler*/ ctx[59](/*i*/ ctx[68], ...args);
    	}

    	let if_block1 = /*allowResizeFromTableHeaders*/ ctx[11] && !/*column*/ ctx[69].disallowResize && create_if_block_4(ctx);

    	return {
    		key: key_1,
    		first: null,
    		c() {
    			div = element("div");
    			if_block0.c();
    			t = space();
    			if (if_block1) if_block1.c();
    			if_block1_anchor = empty();
    			attr(div, "class", "grid-cell svelte-b58rik");
    			set_style(div, "z-index", /*getCellZIndex*/ ctx[34](/*__affixedColumnIndices*/ ctx[1], /*i*/ ctx[68]));

    			set_style(div, "left", getCellLeft({
    				i: /*i*/ ctx[68],
    				columnWidths: /*columnWidths*/ ctx[21],
    				__affixedColumnIndices: /*__affixedColumnIndices*/ ctx[1],
    				__scrollLeft: /*__scrollLeft*/ ctx[8]
    			}) + "px");

    			set_style(div, "width", /*columnWidths*/ ctx[21][/*i*/ ctx[68]] + "px");
    			set_style(div, "height", /*rowHeight*/ ctx[9] + "px");
    			set_style(div, "line-height", /*rowHeight*/ ctx[9] + "px");
    			attr(div, "title", div_title_value = /*column*/ ctx[69].display || "");
    			attr(div, "role", "columnheader");
    			this.first = div;
    		},
    		m(target, anchor, remount) {
    			insert(target, div, anchor);
    			if_blocks[current_block_type_index].m(div, null);
    			insert(target, t, anchor);
    			if (if_block1) if_block1.m(target, anchor);
    			insert(target, if_block1_anchor, anchor);
    			current = true;
    			if (remount) run_all(dispose);

    			dispose = [
    				listen(div, "mousedown", mousedown_handler),
    				action_destroyer(dragCopy_action = dragCopy.call(null, div, /*allowColumnReordering*/ ctx[12]))
    			];
    		},
    		p(new_ctx, dirty) {
    			ctx = new_ctx;
    			let previous_block_index = current_block_type_index;
    			current_block_type_index = select_block_type(ctx);

    			if (current_block_type_index === previous_block_index) {
    				if_blocks[current_block_type_index].p(ctx, dirty);
    			} else {
    				group_outros();

    				transition_out(if_blocks[previous_block_index], 1, 1, () => {
    					if_blocks[previous_block_index] = null;
    				});

    				check_outros();
    				if_block0 = if_blocks[current_block_type_index];

    				if (!if_block0) {
    					if_block0 = if_blocks[current_block_type_index] = if_block_creators[current_block_type_index](ctx);
    					if_block0.c();
    				}

    				transition_in(if_block0, 1);
    				if_block0.m(div, null);
    			}

    			if (!current || dirty[0] & /*__affixedColumnIndices, columns*/ 3) {
    				set_style(div, "z-index", /*getCellZIndex*/ ctx[34](/*__affixedColumnIndices*/ ctx[1], /*i*/ ctx[68]));
    			}

    			if (!current || dirty[0] & /*columns, columnWidths, __affixedColumnIndices, __scrollLeft*/ 2097411) {
    				set_style(div, "left", getCellLeft({
    					i: /*i*/ ctx[68],
    					columnWidths: /*columnWidths*/ ctx[21],
    					__affixedColumnIndices: /*__affixedColumnIndices*/ ctx[1],
    					__scrollLeft: /*__scrollLeft*/ ctx[8]
    				}) + "px");
    			}

    			if (!current || dirty[0] & /*columnWidths, columns*/ 2097153) {
    				set_style(div, "width", /*columnWidths*/ ctx[21][/*i*/ ctx[68]] + "px");
    			}

    			if (!current || dirty[0] & /*rowHeight*/ 512) {
    				set_style(div, "height", /*rowHeight*/ ctx[9] + "px");
    			}

    			if (!current || dirty[0] & /*rowHeight*/ 512) {
    				set_style(div, "line-height", /*rowHeight*/ ctx[9] + "px");
    			}

    			if (!current || dirty[0] & /*columns*/ 1 && div_title_value !== (div_title_value = /*column*/ ctx[69].display || "")) {
    				attr(div, "title", div_title_value);
    			}

    			if (dragCopy_action && is_function(dragCopy_action.update) && dirty[0] & /*allowColumnReordering*/ 4096) dragCopy_action.update.call(null, /*allowColumnReordering*/ ctx[12]);

    			if (/*allowResizeFromTableHeaders*/ ctx[11] && !/*column*/ ctx[69].disallowResize) {
    				if (if_block1) {
    					if_block1.p(ctx, dirty);
    				} else {
    					if_block1 = create_if_block_4(ctx);
    					if_block1.c();
    					if_block1.m(if_block1_anchor.parentNode, if_block1_anchor);
    				}
    			} else if (if_block1) {
    				if_block1.d(1);
    				if_block1 = null;
    			}
    		},
    		i(local) {
    			if (current) return;
    			transition_in(if_block0);
    			current = true;
    		},
    		o(local) {
    			transition_out(if_block0);
    			current = false;
    		},
    		d(detaching) {
    			if (detaching) detach(div);
    			if_blocks[current_block_type_index].d();
    			if (detaching) detach(t);
    			if (if_block1) if_block1.d(detaching);
    			if (detaching) detach(if_block1_anchor);
    			run_all(dispose);
    		}
    	};
    }

    // (957:4) {#if allowColumnAffix}
    function create_if_block_3(ctx) {
    	let div;
    	let dispose;

    	return {
    		c() {
    			div = element("div");
    			attr(div, "class", "column-affix-marker svelte-b58rik");
    			set_style(div, "left", /*columnAffixLineLeft*/ ctx[20] + "px");
    			set_style(div, "height", /*gridSpaceHeight*/ ctx[23] + "px");
    		},
    		m(target, anchor, remount) {
    			insert(target, div, anchor);
    			if (remount) dispose();
    			dispose = listen(div, "mousedown", /*onColumnAffixStart*/ ctx[29]);
    		},
    		p(ctx, dirty) {
    			if (dirty[0] & /*columnAffixLineLeft*/ 1048576) {
    				set_style(div, "left", /*columnAffixLineLeft*/ ctx[20] + "px");
    			}

    			if (dirty[0] & /*gridSpaceHeight*/ 8388608) {
    				set_style(div, "height", /*gridSpaceHeight*/ ctx[23] + "px");
    			}
    		},
    		d(detaching) {
    			if (detaching) detach(div);
    			dispose();
    		}
    	};
    }

    // (972:6) {#if allowResizeFromTableCells}
    function create_if_block_1(ctx) {
    	let each_1_anchor;
    	let each_value_2 = /*columns*/ ctx[0];
    	let each_blocks = [];

    	for (let i = 0; i < each_value_2.length; i += 1) {
    		each_blocks[i] = create_each_block_2(get_each_context_2(ctx, each_value_2, i));
    	}

    	return {
    		c() {
    			for (let i = 0; i < each_blocks.length; i += 1) {
    				each_blocks[i].c();
    			}

    			each_1_anchor = empty();
    		},
    		m(target, anchor) {
    			for (let i = 0; i < each_blocks.length; i += 1) {
    				each_blocks[i].m(target, anchor);
    			}

    			insert(target, each_1_anchor, anchor);
    		},
    		p(ctx, dirty) {
    			if (dirty[0] & /*columnWidths, __affixedColumnIndices, __scrollLeft, __columnHeaderResizeCaptureWidth, columns*/ 2130179 | dirty[1] & /*onColumnResizeStart*/ 2) {
    				each_value_2 = /*columns*/ ctx[0];
    				let i;

    				for (i = 0; i < each_value_2.length; i += 1) {
    					const child_ctx = get_each_context_2(ctx, each_value_2, i);

    					if (each_blocks[i]) {
    						each_blocks[i].p(child_ctx, dirty);
    					} else {
    						each_blocks[i] = create_each_block_2(child_ctx);
    						each_blocks[i].c();
    						each_blocks[i].m(each_1_anchor.parentNode, each_1_anchor);
    					}
    				}

    				for (; i < each_blocks.length; i += 1) {
    					each_blocks[i].d(1);
    				}

    				each_blocks.length = each_value_2.length;
    			}
    		},
    		d(detaching) {
    			destroy_each(each_blocks, detaching);
    			if (detaching) detach(each_1_anchor);
    		}
    	};
    }

    // (974:10) {#if !column.disallowResize}
    function create_if_block_2(ctx) {
    	let div;
    	let dispose;

    	function mousedown_handler_2(...args) {
    		return /*mousedown_handler_2*/ ctx[61](/*i*/ ctx[68], ...args);
    	}

    	return {
    		c() {
    			div = element("div");
    			attr(div, "class", "grid-cell-size-capture svelte-b58rik");

    			set_style(div, "left", getCellLeft({
    				i: /*i*/ ctx[68] + 1,
    				columnWidths: /*columnWidths*/ ctx[21],
    				__affixedColumnIndices: /*__affixedColumnIndices*/ ctx[1],
    				__scrollLeft: /*__scrollLeft*/ ctx[8]
    			}) - Math.floor(/*__columnHeaderResizeCaptureWidth*/ ctx[15] / 2) + "px");

    			set_style(div, "width", /*__columnHeaderResizeCaptureWidth*/ ctx[15] + "px");
    		},
    		m(target, anchor, remount) {
    			insert(target, div, anchor);
    			if (remount) dispose();
    			dispose = listen(div, "mousedown", mousedown_handler_2);
    		},
    		p(new_ctx, dirty) {
    			ctx = new_ctx;

    			if (dirty[0] & /*columnWidths, __affixedColumnIndices, __scrollLeft, __columnHeaderResizeCaptureWidth*/ 2130178) {
    				set_style(div, "left", getCellLeft({
    					i: /*i*/ ctx[68] + 1,
    					columnWidths: /*columnWidths*/ ctx[21],
    					__affixedColumnIndices: /*__affixedColumnIndices*/ ctx[1],
    					__scrollLeft: /*__scrollLeft*/ ctx[8]
    				}) - Math.floor(/*__columnHeaderResizeCaptureWidth*/ ctx[15] / 2) + "px");
    			}

    			if (dirty[0] & /*__columnHeaderResizeCaptureWidth*/ 32768) {
    				set_style(div, "width", /*__columnHeaderResizeCaptureWidth*/ ctx[15] + "px");
    			}
    		},
    		d(detaching) {
    			if (detaching) detach(div);
    			dispose();
    		}
    	};
    }

    // (973:8) {#each columns as column, i}
    function create_each_block_2(ctx) {
    	let if_block_anchor;
    	let if_block = !/*column*/ ctx[69].disallowResize && create_if_block_2(ctx);

    	return {
    		c() {
    			if (if_block) if_block.c();
    			if_block_anchor = empty();
    		},
    		m(target, anchor) {
    			if (if_block) if_block.m(target, anchor);
    			insert(target, if_block_anchor, anchor);
    		},
    		p(ctx, dirty) {
    			if (!/*column*/ ctx[69].disallowResize) {
    				if (if_block) {
    					if_block.p(ctx, dirty);
    				} else {
    					if_block = create_if_block_2(ctx);
    					if_block.c();
    					if_block.m(if_block_anchor.parentNode, if_block_anchor);
    				}
    			} else if (if_block) {
    				if_block.d(1);
    				if_block = null;
    			}
    		},
    		d(detaching) {
    			if (if_block) if_block.d(detaching);
    			if (detaching) detach(if_block_anchor);
    		}
    	};
    }

    // (1013:12) {:else}
    function create_else_block(ctx) {
    	let div;
    	let t_value = (/*row*/ ctx[66].data[/*column*/ ctx[69].dataName] || "") + "";
    	let t;

    	return {
    		c() {
    			div = element("div");
    			t = text(t_value);
    			attr(div, "class", "cell-default svelte-b58rik");
    		},
    		m(target, anchor) {
    			insert(target, div, anchor);
    			append(div, t);
    		},
    		p(ctx, dirty) {
    			if (dirty[0] & /*visibleRows, columns*/ 16777217 && t_value !== (t_value = (/*row*/ ctx[66].data[/*column*/ ctx[69].dataName] || "") + "")) set_data(t, t_value);
    		},
    		i: noop,
    		o: noop,
    		d(detaching) {
    			if (detaching) detach(div);
    		}
    	};
    }

    // (1006:12) {#if column.cellComponent}
    function create_if_block(ctx) {
    	let switch_instance_anchor;
    	let current;
    	var switch_value = /*column*/ ctx[69].cellComponent;

    	function switch_props(ctx) {
    		return {
    			props: {
    				rowNumber: /*row*/ ctx[66].i,
    				column: /*column*/ ctx[69],
    				row: /*row*/ ctx[66]
    			}
    		};
    	}

    	if (switch_value) {
    		var switch_instance = new switch_value(switch_props(ctx));
    		switch_instance.$on("valueupdate", /*onCellUpdated*/ ctx[28]);
    	}

    	return {
    		c() {
    			if (switch_instance) create_component(switch_instance.$$.fragment);
    			switch_instance_anchor = empty();
    		},
    		m(target, anchor) {
    			if (switch_instance) {
    				mount_component(switch_instance, target, anchor);
    			}

    			insert(target, switch_instance_anchor, anchor);
    			current = true;
    		},
    		p(ctx, dirty) {
    			const switch_instance_changes = {};
    			if (dirty[0] & /*visibleRows*/ 16777216) switch_instance_changes.rowNumber = /*row*/ ctx[66].i;
    			if (dirty[0] & /*columns*/ 1) switch_instance_changes.column = /*column*/ ctx[69];
    			if (dirty[0] & /*visibleRows*/ 16777216) switch_instance_changes.row = /*row*/ ctx[66];

    			if (switch_value !== (switch_value = /*column*/ ctx[69].cellComponent)) {
    				if (switch_instance) {
    					group_outros();
    					const old_component = switch_instance;

    					transition_out(old_component.$$.fragment, 1, 0, () => {
    						destroy_component(old_component, 1);
    					});

    					check_outros();
    				}

    				if (switch_value) {
    					switch_instance = new switch_value(switch_props(ctx));
    					switch_instance.$on("valueupdate", /*onCellUpdated*/ ctx[28]);
    					create_component(switch_instance.$$.fragment);
    					transition_in(switch_instance.$$.fragment, 1);
    					mount_component(switch_instance, switch_instance_anchor.parentNode, switch_instance_anchor);
    				} else {
    					switch_instance = null;
    				}
    			} else if (switch_value) {
    				switch_instance.$set(switch_instance_changes);
    			}
    		},
    		i(local) {
    			if (current) return;
    			if (switch_instance) transition_in(switch_instance.$$.fragment, local);
    			current = true;
    		},
    		o(local) {
    			if (switch_instance) transition_out(switch_instance.$$.fragment, local);
    			current = false;
    		},
    		d(detaching) {
    			if (detaching) detach(switch_instance_anchor);
    			if (switch_instance) destroy_component(switch_instance, detaching);
    		}
    	};
    }

    // (999:8) {#each columns as column, j}
    function create_each_block_1(ctx) {
    	let div;
    	let current_block_type_index;
    	let if_block;
    	let current;
    	const if_block_creators = [create_if_block, create_else_block];
    	const if_blocks = [];

    	function select_block_type_1(ctx, dirty) {
    		if (/*column*/ ctx[69].cellComponent) return 0;
    		return 1;
    	}

    	current_block_type_index = select_block_type_1(ctx);
    	if_block = if_blocks[current_block_type_index] = if_block_creators[current_block_type_index](ctx);

    	return {
    		c() {
    			div = element("div");
    			if_block.c();
    			attr(div, "class", "grid-cell svelte-b58rik");
    			set_style(div, "z-index", /*getCellZIndex*/ ctx[34](/*__affixedColumnIndices*/ ctx[1], /*j*/ ctx[71]));

    			set_style(div, "left", getCellLeft({
    				i: /*j*/ ctx[71],
    				columnWidths: /*columnWidths*/ ctx[21],
    				__affixedColumnIndices: /*__affixedColumnIndices*/ ctx[1],
    				__scrollLeft: /*__scrollLeft*/ ctx[8]
    			}) + "px");

    			set_style(div, "height", /*rowHeight*/ ctx[9] + "px");
    			set_style(div, "line-height", /*rowHeight*/ ctx[9] + "px");
    			set_style(div, "width", /*columnWidths*/ ctx[21][/*j*/ ctx[71]] + "px");
    			attr(div, "role", "cell");
    		},
    		m(target, anchor) {
    			insert(target, div, anchor);
    			if_blocks[current_block_type_index].m(div, null);
    			current = true;
    		},
    		p(ctx, dirty) {
    			let previous_block_index = current_block_type_index;
    			current_block_type_index = select_block_type_1(ctx);

    			if (current_block_type_index === previous_block_index) {
    				if_blocks[current_block_type_index].p(ctx, dirty);
    			} else {
    				group_outros();

    				transition_out(if_blocks[previous_block_index], 1, 1, () => {
    					if_blocks[previous_block_index] = null;
    				});

    				check_outros();
    				if_block = if_blocks[current_block_type_index];

    				if (!if_block) {
    					if_block = if_blocks[current_block_type_index] = if_block_creators[current_block_type_index](ctx);
    					if_block.c();
    				}

    				transition_in(if_block, 1);
    				if_block.m(div, null);
    			}

    			if (!current || dirty[0] & /*__affixedColumnIndices*/ 2) {
    				set_style(div, "z-index", /*getCellZIndex*/ ctx[34](/*__affixedColumnIndices*/ ctx[1], /*j*/ ctx[71]));
    			}

    			if (!current || dirty[0] & /*columnWidths, __affixedColumnIndices, __scrollLeft*/ 2097410) {
    				set_style(div, "left", getCellLeft({
    					i: /*j*/ ctx[71],
    					columnWidths: /*columnWidths*/ ctx[21],
    					__affixedColumnIndices: /*__affixedColumnIndices*/ ctx[1],
    					__scrollLeft: /*__scrollLeft*/ ctx[8]
    				}) + "px");
    			}

    			if (!current || dirty[0] & /*rowHeight*/ 512) {
    				set_style(div, "height", /*rowHeight*/ ctx[9] + "px");
    			}

    			if (!current || dirty[0] & /*rowHeight*/ 512) {
    				set_style(div, "line-height", /*rowHeight*/ ctx[9] + "px");
    			}

    			if (!current || dirty[0] & /*columnWidths*/ 2097152) {
    				set_style(div, "width", /*columnWidths*/ ctx[21][/*j*/ ctx[71]] + "px");
    			}
    		},
    		i(local) {
    			if (current) return;
    			transition_in(if_block);
    			current = true;
    		},
    		o(local) {
    			transition_out(if_block);
    			current = false;
    		},
    		d(detaching) {
    			if (detaching) detach(div);
    			if_blocks[current_block_type_index].d();
    		}
    	};
    }

    // (991:4) {#each visibleRows as row, i}
    function create_each_block(ctx) {
    	let div;
    	let t;
    	let div_aria_rowindex_value;
    	let current;
    	let dispose;
    	let each_value_1 = /*columns*/ ctx[0];
    	let each_blocks = [];

    	for (let i = 0; i < each_value_1.length; i += 1) {
    		each_blocks[i] = create_each_block_1(get_each_context_1(ctx, each_value_1, i));
    	}

    	const out = i => transition_out(each_blocks[i], 1, 1, () => {
    		each_blocks[i] = null;
    	});

    	function click_handler(...args) {
    		return /*click_handler*/ ctx[62](/*row*/ ctx[66], ...args);
    	}

    	return {
    		c() {
    			div = element("div");

    			for (let i = 0; i < each_blocks.length; i += 1) {
    				each_blocks[i].c();
    			}

    			t = space();
    			attr(div, "class", "grid-row svelte-b58rik");
    			set_style(div, "top", /*getRowTop*/ ctx[35](/*row*/ ctx[66].i, /*rowHeight*/ ctx[9]) + "px");
    			set_style(div, "height", /*rowHeight*/ ctx[9] + "px");
    			set_style(div, "width", /*gridSpaceWidth*/ ctx[22] + "px");
    			attr(div, "role", "row");
    			attr(div, "aria-rowindex", div_aria_rowindex_value = /*row*/ ctx[66].i);
    		},
    		m(target, anchor, remount) {
    			insert(target, div, anchor);

    			for (let i = 0; i < each_blocks.length; i += 1) {
    				each_blocks[i].m(div, null);
    			}

    			append(div, t);
    			current = true;
    			if (remount) dispose();
    			dispose = listen(div, "click", click_handler);
    		},
    		p(new_ctx, dirty) {
    			ctx = new_ctx;

    			if (dirty[0] & /*__affixedColumnIndices, columnWidths, __scrollLeft, rowHeight, columns, visibleRows, onCellUpdated*/ 287310595 | dirty[1] & /*getCellZIndex*/ 8) {
    				each_value_1 = /*columns*/ ctx[0];
    				let i;

    				for (i = 0; i < each_value_1.length; i += 1) {
    					const child_ctx = get_each_context_1(ctx, each_value_1, i);

    					if (each_blocks[i]) {
    						each_blocks[i].p(child_ctx, dirty);
    						transition_in(each_blocks[i], 1);
    					} else {
    						each_blocks[i] = create_each_block_1(child_ctx);
    						each_blocks[i].c();
    						transition_in(each_blocks[i], 1);
    						each_blocks[i].m(div, t);
    					}
    				}

    				group_outros();

    				for (i = each_value_1.length; i < each_blocks.length; i += 1) {
    					out(i);
    				}

    				check_outros();
    			}

    			if (!current || dirty[0] & /*visibleRows, rowHeight*/ 16777728) {
    				set_style(div, "top", /*getRowTop*/ ctx[35](/*row*/ ctx[66].i, /*rowHeight*/ ctx[9]) + "px");
    			}

    			if (!current || dirty[0] & /*rowHeight*/ 512) {
    				set_style(div, "height", /*rowHeight*/ ctx[9] + "px");
    			}

    			if (!current || dirty[0] & /*gridSpaceWidth*/ 4194304) {
    				set_style(div, "width", /*gridSpaceWidth*/ ctx[22] + "px");
    			}

    			if (!current || dirty[0] & /*visibleRows*/ 16777216 && div_aria_rowindex_value !== (div_aria_rowindex_value = /*row*/ ctx[66].i)) {
    				attr(div, "aria-rowindex", div_aria_rowindex_value);
    			}
    		},
    		i(local) {
    			if (current) return;

    			for (let i = 0; i < each_value_1.length; i += 1) {
    				transition_in(each_blocks[i]);
    			}

    			current = true;
    		},
    		o(local) {
    			each_blocks = each_blocks.filter(Boolean);

    			for (let i = 0; i < each_blocks.length; i += 1) {
    				transition_out(each_blocks[i]);
    			}

    			current = false;
    		},
    		d(detaching) {
    			if (detaching) detach(div);
    			destroy_each(each_blocks, detaching);
    			dispose();
    		}
    	};
    }

    function create_fragment(ctx) {
    	let div5;
    	let t0;
    	let t1;
    	let div1;
    	let div0;
    	let each_blocks_1 = [];
    	let each0_lookup = new Map();
    	let t2;
    	let div4;
    	let t3;
    	let div2;
    	let t4;
    	let div3;
    	let t5;
    	let div4_resize_listener;
    	let div5_class_value;
    	let current;
    	let dispose;
    	let if_block0 = (/*__resizing*/ ctx[5] || /*__columnDragging*/ ctx[4] || /*__affixingColumn*/ ctx[3]) && create_if_block_7(ctx);
    	let if_block1 = /*__affixingRow*/ ctx[2] && create_if_block_6(ctx);
    	let each_value_3 = /*columns*/ ctx[0];
    	const get_key = ctx => /*i*/ ctx[68];

    	for (let i = 0; i < each_value_3.length; i += 1) {
    		let child_ctx = get_each_context_3(ctx, each_value_3, i);
    		let key = get_key(child_ctx);
    		each0_lookup.set(key, each_blocks_1[i] = create_each_block_3(key, child_ctx));
    	}

    	let if_block2 = /*allowColumnAffix*/ ctx[13] && create_if_block_3(ctx);
    	let if_block3 = /*allowResizeFromTableCells*/ ctx[10] && create_if_block_1(ctx);
    	let each_value = /*visibleRows*/ ctx[24];
    	let each_blocks = [];

    	for (let i = 0; i < each_value.length; i += 1) {
    		each_blocks[i] = create_each_block(get_each_context(ctx, each_value, i));
    	}

    	const out = i => transition_out(each_blocks[i], 1, 1, () => {
    		each_blocks[i] = null;
    	});

    	return {
    		c() {
    			div5 = element("div");
    			if (if_block0) if_block0.c();
    			t0 = space();
    			if (if_block1) if_block1.c();
    			t1 = space();
    			div1 = element("div");
    			div0 = element("div");

    			for (let i = 0; i < each_blocks_1.length; i += 1) {
    				each_blocks_1[i].c();
    			}

    			t2 = space();
    			div4 = element("div");
    			if (if_block2) if_block2.c();
    			t3 = space();
    			div2 = element("div");
    			t4 = space();
    			div3 = element("div");
    			if (if_block3) if_block3.c();
    			t5 = space();

    			for (let i = 0; i < each_blocks.length; i += 1) {
    				each_blocks[i].c();
    			}

    			attr(div0, "class", "grid-header-row svelte-b58rik");
    			set_style(div0, "left", "-" + /*__scrollLeft*/ ctx[8] + "px");
    			set_style(div0, "height", /*rowHeight*/ ctx[9] + "px");
    			set_style(div0, "width", /*gridSpaceWidth*/ ctx[22] + "px");
    			attr(div0, "role", "row");
    			attr(div1, "class", "grid-headers svelte-b58rik");
    			set_style(div1, "height", /*rowHeight*/ ctx[9] + "px");
    			attr(div1, "rolw", "rowgroup");
    			attr(div2, "class", "row-affix-marker svelte-b58rik");
    			set_style(div2, "top", /*__rowAffixLineTop*/ ctx[17] + "px");
    			set_style(div2, "width", /*gridSpaceWidth*/ ctx[22] + "px");
    			attr(div3, "class", "grid-space svelte-b58rik");
    			set_style(div3, "width", /*gridSpaceWidth*/ ctx[22] + "px");
    			set_style(div3, "height", /*gridSpaceHeight*/ ctx[23] + "px");
    			attr(div4, "class", "grid-inner svelte-b58rik");
    			set_style(div4, "height", "100%");
    			attr(div4, "role", "rowgroup");
    			add_render_callback(() => /*div4_elementresize_handler*/ ctx[64].call(div4));

    			attr(div5, "class", div5_class_value = "data-grid-wrapper " + (/*__resizing*/ ctx[5] || /*__columnDragging*/ ctx[4]
    			? "resizing"
    			: "") + " svelte-b58rik");

    			set_style(div5, "padding-top", /*rowHeight*/ ctx[9] + "px");
    			attr(div5, "role", "table");
    		},
    		m(target, anchor, remount) {
    			insert(target, div5, anchor);
    			if (if_block0) if_block0.m(div5, null);
    			append(div5, t0);
    			if (if_block1) if_block1.m(div5, null);
    			append(div5, t1);
    			append(div5, div1);
    			append(div1, div0);

    			for (let i = 0; i < each_blocks_1.length; i += 1) {
    				each_blocks_1[i].m(div0, null);
    			}

    			append(div5, t2);
    			append(div5, div4);
    			if (if_block2) if_block2.m(div4, null);
    			append(div4, t3);
    			append(div4, div2);
    			append(div4, t4);
    			append(div4, div3);
    			if (if_block3) if_block3.m(div3, null);
    			append(div4, t5);

    			for (let i = 0; i < each_blocks.length; i += 1) {
    				each_blocks[i].m(div4, null);
    			}

    			/*div4_binding*/ ctx[63](div4);
    			div4_resize_listener = add_resize_listener(div4, /*div4_elementresize_handler*/ ctx[64].bind(div4));
    			/*div5_binding*/ ctx[65](div5);
    			current = true;
    			if (remount) run_all(dispose);

    			dispose = [
    				listen(window_1, "mouseup", /*onMouseUp*/ ctx[27]),
    				listen(window_1, "mousemove", /*onMouseMove*/ ctx[26]),
    				listen(window_1, "keydown", /*onWindowKeyDown*/ ctx[25]),
    				listen(div2, "mousedown", /*onRowAffixStart*/ ctx[30]),
    				listen(div4, "scroll", /*onScroll*/ ctx[33])
    			];
    		},
    		p(ctx, dirty) {
    			if (/*__resizing*/ ctx[5] || /*__columnDragging*/ ctx[4] || /*__affixingColumn*/ ctx[3]) {
    				if (if_block0) {
    					if_block0.p(ctx, dirty);
    				} else {
    					if_block0 = create_if_block_7(ctx);
    					if_block0.c();
    					if_block0.m(div5, t0);
    				}
    			} else if (if_block0) {
    				if_block0.d(1);
    				if_block0 = null;
    			}

    			if (/*__affixingRow*/ ctx[2]) {
    				if (if_block1) {
    					if_block1.p(ctx, dirty);
    				} else {
    					if_block1 = create_if_block_6(ctx);
    					if_block1.c();
    					if_block1.m(div5, t1);
    				}
    			} else if (if_block1) {
    				if_block1.d(1);
    				if_block1 = null;
    			}

    			if (dirty[0] & /*columns, columnWidths, __affixedColumnIndices, __scrollLeft, __columnHeaderResizeCaptureWidth, allowResizeFromTableHeaders, rowHeight, allowColumnReordering*/ 2136835 | dirty[1] & /*onColumnResizeStart, getCellZIndex, onColumnDragStart*/ 11) {
    				const each_value_3 = /*columns*/ ctx[0];
    				group_outros();
    				each_blocks_1 = update_keyed_each(each_blocks_1, dirty, get_key, 1, ctx, each_value_3, each0_lookup, div0, outro_and_destroy_block, create_each_block_3, null, get_each_context_3);
    				check_outros();
    			}

    			if (!current || dirty[0] & /*__scrollLeft*/ 256) {
    				set_style(div0, "left", "-" + /*__scrollLeft*/ ctx[8] + "px");
    			}

    			if (!current || dirty[0] & /*rowHeight*/ 512) {
    				set_style(div0, "height", /*rowHeight*/ ctx[9] + "px");
    			}

    			if (!current || dirty[0] & /*gridSpaceWidth*/ 4194304) {
    				set_style(div0, "width", /*gridSpaceWidth*/ ctx[22] + "px");
    			}

    			if (!current || dirty[0] & /*rowHeight*/ 512) {
    				set_style(div1, "height", /*rowHeight*/ ctx[9] + "px");
    			}

    			if (/*allowColumnAffix*/ ctx[13]) {
    				if (if_block2) {
    					if_block2.p(ctx, dirty);
    				} else {
    					if_block2 = create_if_block_3(ctx);
    					if_block2.c();
    					if_block2.m(div4, t3);
    				}
    			} else if (if_block2) {
    				if_block2.d(1);
    				if_block2 = null;
    			}

    			if (!current || dirty[0] & /*__rowAffixLineTop*/ 131072) {
    				set_style(div2, "top", /*__rowAffixLineTop*/ ctx[17] + "px");
    			}

    			if (!current || dirty[0] & /*gridSpaceWidth*/ 4194304) {
    				set_style(div2, "width", /*gridSpaceWidth*/ ctx[22] + "px");
    			}

    			if (/*allowResizeFromTableCells*/ ctx[10]) {
    				if (if_block3) {
    					if_block3.p(ctx, dirty);
    				} else {
    					if_block3 = create_if_block_1(ctx);
    					if_block3.c();
    					if_block3.m(div3, null);
    				}
    			} else if (if_block3) {
    				if_block3.d(1);
    				if_block3 = null;
    			}

    			if (!current || dirty[0] & /*gridSpaceWidth*/ 4194304) {
    				set_style(div3, "width", /*gridSpaceWidth*/ ctx[22] + "px");
    			}

    			if (!current || dirty[0] & /*gridSpaceHeight*/ 8388608) {
    				set_style(div3, "height", /*gridSpaceHeight*/ ctx[23] + "px");
    			}

    			if (dirty[0] & /*visibleRows, rowHeight, gridSpaceWidth, onRowClick, columns, __affixedColumnIndices, columnWidths, __scrollLeft, onCellUpdated*/ 291521283 | dirty[1] & /*getRowTop, getCellZIndex*/ 24) {
    				each_value = /*visibleRows*/ ctx[24];
    				let i;

    				for (i = 0; i < each_value.length; i += 1) {
    					const child_ctx = get_each_context(ctx, each_value, i);

    					if (each_blocks[i]) {
    						each_blocks[i].p(child_ctx, dirty);
    						transition_in(each_blocks[i], 1);
    					} else {
    						each_blocks[i] = create_each_block(child_ctx);
    						each_blocks[i].c();
    						transition_in(each_blocks[i], 1);
    						each_blocks[i].m(div4, null);
    					}
    				}

    				group_outros();

    				for (i = each_value.length; i < each_blocks.length; i += 1) {
    					out(i);
    				}

    				check_outros();
    			}

    			if (!current || dirty[0] & /*__resizing, __columnDragging*/ 48 && div5_class_value !== (div5_class_value = "data-grid-wrapper " + (/*__resizing*/ ctx[5] || /*__columnDragging*/ ctx[4]
    			? "resizing"
    			: "") + " svelte-b58rik")) {
    				attr(div5, "class", div5_class_value);
    			}

    			if (!current || dirty[0] & /*rowHeight*/ 512) {
    				set_style(div5, "padding-top", /*rowHeight*/ ctx[9] + "px");
    			}
    		},
    		i(local) {
    			if (current) return;

    			for (let i = 0; i < each_value_3.length; i += 1) {
    				transition_in(each_blocks_1[i]);
    			}

    			for (let i = 0; i < each_value.length; i += 1) {
    				transition_in(each_blocks[i]);
    			}

    			current = true;
    		},
    		o(local) {
    			for (let i = 0; i < each_blocks_1.length; i += 1) {
    				transition_out(each_blocks_1[i]);
    			}

    			each_blocks = each_blocks.filter(Boolean);

    			for (let i = 0; i < each_blocks.length; i += 1) {
    				transition_out(each_blocks[i]);
    			}

    			current = false;
    		},
    		d(detaching) {
    			if (detaching) detach(div5);
    			if (if_block0) if_block0.d();
    			if (if_block1) if_block1.d();

    			for (let i = 0; i < each_blocks_1.length; i += 1) {
    				each_blocks_1[i].d();
    			}

    			if (if_block2) if_block2.d();
    			if (if_block3) if_block3.d();
    			destroy_each(each_blocks, detaching);
    			/*div4_binding*/ ctx[63](null);
    			div4_resize_listener.cancel();
    			/*div5_binding*/ ctx[65](null);
    			run_all(dispose);
    		}
    	};
    }

    const MIN_COLUMN_SIZE = 30;

    /**
     * Computes the 'left' value for a grid-cell.
     * @param {Number} i The cell index
     * @param {Array} columnWidths The array of column widths in order
     * @returns {Number}
     */
    function getCellLeft({ i, columnWidths, __affixedColumnIndices, __scrollLeft }) {
    	if (__affixedColumnIndices.indexOf(i) >= 0) {
    		if (i === 0) {
    			return __scrollLeft;
    		}

    		let left = __scrollLeft;

    		for (let j = i - 1; j >= 0; j--) {
    			left += columnWidths[j];
    		}

    		return left;
    	}

    	let left = 0;

    	for (let j = 0; j < i; j++) {
    		left += columnWidths[j];
    	}

    	return left;
    }

    /**
     * Gets the closest column index given an x offset
     * @param {Number} x The x offset
     * @param {Array} columnWidths Array of column widths
     * @param {Array} __affixedColumnIndices Array of column indices that have been affixed
     * @param {Number} __scrollLeft The scrollLeft value of the scrollable container
     * @returns {Number}
     */
    function getClosestIndex(x, columnWidths, __affixedColumnIndices, __scrollLeft) {
    	let closest = 0;

    	for (let i = 0; i < columnWidths.length; i++) {
    		const left = getCellLeft({
    			i,
    			columnWidths,
    			__affixedColumnIndices,
    			__scrollLeft
    		}) + Math.floor(columnWidths[i] / 2);

    		if (left < x) {
    			closest = i + 1;
    		}
    	}

    	// special handling required when there are affixed columns, because
    	// if the grid is scrolled horizontally to the right, we want to have this function return the closest
    	// affixed column, rather than any columns that might be closer to x but are being overlapped by an affixed column
    	if (__affixedColumnIndices.length > 0) {
    		const firstAffixedLeft = getCellLeft({
    			i: __affixedColumnIndices[0],
    			columnWidths,
    			__affixedColumnIndices,
    			__scrollLeft
    		});

    		const lastAffixedLeft = getCellLeft({
    			i: __affixedColumnIndices[__affixedColumnIndices.length - 1],
    			columnWidths,
    			__affixedColumnIndices,
    			__scrollLeft
    		});

    		const lastAffixedRight = lastAffixedLeft + columnWidths[__affixedColumnIndices[__affixedColumnIndices.length - 1]];

    		const closestLeft = getCellLeft({
    			i: closest,
    			columnWidths,
    			__affixedColumnIndices,
    			__scrollLeft
    		});

    		if (closestLeft > firstAffixedLeft && closestLeft < lastAffixedRight) {
    			if (closestLeft < lastAffixedRight && closestLeft > lastAffixedLeft) {
    				closest = __affixedColumnIndices[__affixedColumnIndices.length - 1];
    			} else {
    				for (let i = 0; i < __affixedColumnIndices.length; i++) {
    					const left = getCellLeft({
    						i: __affixedColumnIndices[i],
    						columnWidths,
    						__affixedColumnIndices,
    						__scrollLeft
    					}) + Math.floor(columnWidths[__affixedColumnIndices[i]] / 2);

    					if (left < x) {
    						closest = __affixedColumnIndices[i] + 1;
    					}
    				}
    			}
    		}
    	}

    	return closest;
    }

    function getBodyScrollTop() {
    	return window.pageYOffset || (document.documentElement.clientHeight
    	? document.documentElement.scrollTop
    	: document.body.scrollTop);
    }

    /**
     * TODO handle svelte 2's onupdate
     **/
    // onupdate(({ changed, current, previous }) => {
    //   // Record the change in onupdate to allow the DOM to change before doing the deep diff
    //   if (changed.rows && previous && previous.rows && previous.rows.length > 0) {
    //     if (!current.skipRecord) {
    //       this.editHistory.recordChange(current.rows);
    //     } else {
    //       this.set({ skipRecord: false });
    //     }
    //   }
    // });
    /**
     * This action creates a half-opaque 'ghost' column header to visualize dragging a column into a different position
     * This is put in an action because ultimately the ghost image has nothing to do with the actual column index move
     */
    function dragCopy(node, enabled) {
    	let copy = null;
    	let dragging = false;
    	let offsetX = 0;

    	function onWindowMouseMove(event) {
    		if (!dragging) {
    			return;
    		}

    		copy.style.left = event.pageX - offsetX + "px";
    	}

    	function onWindowMouseUp(event) {
    		if (!dragging || event.which !== 1) {
    			return;
    		}

    		dragging = false;
    		document.body.removeChild(copy);
    		copy = null;
    	}

    	function onNodeMouseDown(event) {
    		if (event.which !== 1) {
    			return;
    		}

    		dragging = true;

    		if (copy) {
    			document.body.removeChild(copy);
    		}

    		copy = createCopy();
    		offsetX = event.offsetX;
    		copy.style.top = node.getBoundingClientRect().top + getBodyScrollTop() + "px";
    		copy.style.left = event.pageX - offsetX + "px";
    		document.body.appendChild(copy);
    	}

    	function createCopy() {
    		const copy = document.createElement("div");
    		copy.innerHTML = node.innerHTML;
    		const { width, height, textAlign, fontWeight } = getComputedStyle(node);
    		copy.style.width = width;
    		copy.style.height = height;
    		copy.style.maxHeight = height;
    		copy.style.textAlign = textAlign;
    		copy.style.fontWeight = fontWeight;
    		copy.style.position = "absolute";
    		copy.style.opacity = "0.5";
    		copy.style.pointerEvents = "none";
    		copy.style.overflow = "hidden";
    		copy.style.background = "#dddddd";
    		copy.style["z-index"] = "99999";
    		return copy;
    	}

    	function attachEvents() {
    		window.addEventListener("mousemove", onWindowMouseMove);
    		window.addEventListener("mouseup", onWindowMouseUp);
    		node.addEventListener("mousedown", onNodeMouseDown);
    	}

    	function detachEvents() {
    		window.removeEventListener("mousemove", onWindowMouseMove);
    		window.removeEventListener("mouseup", onWindowMouseUp);
    		node.removeEventListener("mousedown", onNodeMouseDown);
    	}

    	if (enabled) {
    		attachEvents();
    	}

    	return {
    		destroy() {
    			detachEvents();
    		},
    		update(enabled) {
    			if (enabled) {
    				attachEvents();
    			} else {
    				detachEvents();
    			}
    		}
    	};
    }

    function instance($$self, $$props, $$invalidate) {
    	const dispatch = createEventDispatcher();
    	let wrapper;
    	let tableSpace;
    	let editHistory = null;
    	let { rows = [] } = $$props; // Rows to display
    	let { columns = [] } = $$props; // Array of column definitions: { display: '', dataName: ''}, where display is what the display value is and dataName is what the key on the row object is
    	let { rowHeight = 24 } = $$props; // Row height in pixels
    	let { allowResizeFromTableCells = false } = $$props; // Allow the user to click on table cell borders to resize columns
    	let { allowResizeFromTableHeaders = true } = $$props; // Allow the user to clikc on table header borders to resize columns
    	let { allowColumnReordering = true } = $$props; // Allow the user to drag column headers to reorder columns
    	let { allowColumnAffix = true } = $$props; // Alow the user to affix columns to the left of the grid
    	let { onRowClick } = $$props;
    	let { __extraRows = 0 } = $$props; // Number of extra rows to render beyond what is visible in the scrollable area
    	let { __columnHeaderResizeCaptureWidth = 20 } = $$props; // The width of the area on column borders that can be clicked to resize the column
    	let { __affixedRowIndices = [] } = $$props; // DO NOT MODIFY DIRECTLY. The row indices to affix to the top of the grid
    	let { __affixedColumnIndices = [] } = $$props; // DO NOT MODIFY DIRECTLY. The column indices to affix to the left side of the grid
    	let { __affixingRow = false } = $$props; // DO NOT MODIFY DIRECTLY. Whether a row affix operation is in progress
    	let { __affixingColumn = false } = $$props; // DO NOT MODIFY DIRECTLY. Whether a column affix operation is in progress
    	let { __rowActionLineTop = 0 } = $$props; // DO NOT MODIFY DIRECTLY. The 'top' position of the row action line
    	let { __rowAffixLineTop = 0 } = $$props; // DO NOT MODIFY DIRECTLY. The 'top' position of the row affix line
    	let { __columnAffixLineLeft = 0 } = $$props; // DO NOT MODIFY DIRECTLY. The 'left' position of the column affix line
    	let { __columnDragging = false } = $$props; // DO NOT MODIFY DIRECTLY. Whether a column is being dragged
    	let { __columnIndexBeingDragged = null } = $$props; // DO NOT MODIFY DIRECTLY. The column index that is being dragged
    	let { __columnDragOffsetX = 0 } = $$props; // DO NOT MODIFY DIRECTLY. The X offset of where the user clicked on the column header
    	let { __resizing = false } = $$props; // DO NOT MODIFY DIRECTLY. Whether or not a column is currently being resized
    	let { __columnIndexBeingResized = null } = $$props; // DO NOT MODIFY DIRECTLY. The column index being resized
    	let { __columnActionLineLeft = 0 } = $$props; // DO NOT MODIFY DIRECTLY. The 'left' position of the action line
    	let { __innerOffsetHeight = 0 } = $$props; // DO NOT MODIFY DIRECTLY. The height of the scrollable area on screen
    	let { __scrollTop = 0 } = $$props; // DO NOT MODIFY DIRECTLY. The scrollTop position of the scrollable area
    	let { __scrollLeft = 0 } = $$props; // DO NOT MODIFY DIRECTLY. The scrollLeft position of the scrollable area
    	let { __scrolledAllTheWayToTheRight = false } = $$props; // DO NOT MODIFY DIRECTLY. Whether the container is scrolled all the way to the right as of the last onscroll event

    	onMount(() => {
    		editHistory = new EditHistory(rows);
    	});

    	function onWindowKeyDown(event) {
    		if (event.ctrlKey) {
    			if (event.keyCode === 90) {
    				undo();
    				event.preventDefault();
    			}

    			if (event.keyCode === 89) {
    				redo();
    				event.preventDefault();
    			}
    		}
    	}

    	/**
     * Event handler for window's mousemove event
     * @param {MouseEvent} event The MouseEvent object
     */
    	function onMouseMove(event) {
    		onColumnDragMouseMove(event);
    		onColumnResizeMouseMove(event);
    		onColumnAffixMouseMove(event);
    	}

    	/**
     * Event handler for window's mouseup event
     * @param {MouseEvent} event The MouseEvent object
     */
    	function onMouseUp(event) {
    		onColumnDragEnd(event);
    		onColumnResizeEnd();
    		onRowAffixEnd();
    		onColumnAffixEnd();
    	}

    	/**
     * Event handler for when a value has been updated
     * @param {Object} event Event object with row and column objects
     */
    	function onCellUpdated(event) {
    		$$invalidate(36, rows[event.detail.rowNumber][event.detail.column.dataName] = event.detail.value, rows);
    		dispatch("valueUpdated", event);
    	}

    	/**
     * Applies the most recent backward change
     */
    	function undo() {
    		const eRows = editHistory.undo();

    		if (eRows) {
    			$$invalidate(36, rows = eRows);
    		}
    	}

    	/**
     * Applies the most recent forward change
     */
    	function redo() {
    		const eRows = editHistory.redo();

    		if (eRows) {
    			$$invalidate(36, rows = eRows);
    		}
    	}

    	/**
     * Event handler for starting column affix operation
     */
    	function onColumnAffixStart(event) {
    		// left click only
    		if (event.which !== 1) {
    			return;
    		}

    		if (__affixedColumnIndices.length > 0) {
    			$$invalidate(19, tableSpace.scrollLeft = 0, tableSpace);
    			$$invalidate(3, __affixingColumn = true);
    		} else {
    			$$invalidate(3, __affixingColumn = true);
    		}
    	}

    	/**
     * Event handler for mousemove column affix operation
     */
    	function onColumnAffixMouseMove(event) {
    		if (!__affixingColumn) {
    			return;
    		}

    		if (event.which !== 1) {
    			onColumnAffixEnd();
    			return;
    		}

    		const { left: wrapperPageX } = wrapper.getBoundingClientRect();
    		const offsetPoint = event.pageX - wrapperPageX + __scrollLeft;
    		const idx = getClosestIndex(offsetPoint, columnWidths, __affixedColumnIndices, __scrollLeft);
    		const indices = [];

    		for (let i = 0; i < idx; i++) {
    			indices.push(i);
    		}

    		$$invalidate(6, __columnActionLineLeft = offsetPoint);
    		$$invalidate(1, __affixedColumnIndices = indices);
    		event.preventDefault();
    	} // check to see if horizontal scroll position doesn't match where the

    	/**
     * Event handler for ending column affix operation
     */
    	function onColumnAffixEnd(event) {
    		$$invalidate(3, __affixingColumn = false);
    	}

    	/**
     * Event handler for starting row affix operation
     */
    	function onRowAffixStart(event) {
    		$$invalidate(2, __affixingRow = true);
    	}

    	/**
     * Event handler for mousemove row affix operation
     */
    	function onRowAffixMouseMove(event) {
    		if (!__affixingRow) {
    			return;
    		}
    	}

    	/**
     * Event handler for ending row affix operation
     */
    	function onRowAffixEnd(event) {
    		$$invalidate(2, __affixingRow = false);
    	}

    	/**
     * Event handler for column dragging
     */
    	function onColumnDragStart(event, columnIndex) {
    		if (event.which !== 1) {
    			return;
    		}

    		// if the developer has disabled column reordering, don't begin a reorder
    		if (!allowColumnReordering) {
    			return;
    		}

    		$$invalidate(4, __columnDragging = true);
    		$$invalidate(37, __columnIndexBeingDragged = columnIndex);
    		$$invalidate(38, __columnDragOffsetX = event.offsetX);

    		$$invalidate(6, __columnActionLineLeft = getCellLeft({
    			i: columnIndex,
    			columnWidths,
    			__scrollLeft,
    			__affixedColumnIndices
    		}) - __scrollLeft);
    	}

    	function onColumnDragMouseMove(event) {
    		if (!__columnDragging) {
    			return;
    		}

    		// if user is no longer pressing the left mouse button and we are out of sync
    		// with __columnDragging because mouseup didn't fire, finish the reorder
    		if (event.which !== 1) {
    			onColumnDragEnd(event);
    			return;
    		}

    		const { left: wrapperPageX } = wrapper.getBoundingClientRect();

    		// change the position of the action line to the closest column index under the mouse
    		const offsetPoint = event.pageX - wrapperPageX + __scrollLeft - __columnDragOffsetX;

    		const idx = getClosestIndex(offsetPoint, columnWidths, __affixedColumnIndices, __scrollLeft);

    		$$invalidate(6, __columnActionLineLeft = getCellLeft({
    			i: idx,
    			columnWidths,
    			__affixedColumnIndices,
    			__scrollLeft
    		}) - __scrollLeft);
    	}

    	/**
     * Window mouseup handler for column dragging
     */
    	function onColumnDragEnd(event) {
    		// user might try to be clever and middle-click to scroll horizontally while dragging a column
    		// don't stop the drag for middle clicks
    		if (event.which !== 1) {
    			return;
    		}

    		// if a column isn't being dragged, don't reorder anything
    		if (!__columnDragging) {
    			return;
    		}

    		const { left: wrapperPageX } = wrapper.getBoundingClientRect();
    		const offsetPoint = event.pageX - wrapperPageX + __scrollLeft - __columnDragOffsetX;

    		// move column object to its new position in the array based off the mouse position and scroll position
    		const newIdx = getClosestIndex(offsetPoint, columnWidths, __affixedColumnIndices, __scrollLeft);

    		let mColumns = columns;
    		mColumns.splice(newIdx > __columnIndexBeingDragged ? newIdx - 1 : newIdx, 0, mColumns.splice(__columnIndexBeingDragged, 1)[0]);
    		$$invalidate(0, columns = mColumns);

    		// delay firing of event so that new column order is accessible when handlers are called
    		setTimeout(() => dispatch("columnOrderUpdated"), 0);

    		$$invalidate(4, __columnDragging = false);
    		$$invalidate(38, __columnDragOffsetX = 0);
    		$$invalidate(37, __columnIndexBeingDragged = null);
    	}

    	/**
     * Mousedown handler for column resizing
     */
    	function onColumnResizeStart(event, columnIndex) {
    		// left click only
    		if (event.which !== 1) {
    			return;
    		}

    		const { left: wrapperPageX } = wrapper.getBoundingClientRect();
    		$$invalidate(5, __resizing = true);
    		$$invalidate(6, __columnActionLineLeft = event.pageX - wrapperPageX - __scrollLeft);
    		$$invalidate(39, __columnIndexBeingResized = columnIndex);
    		event.stopPropagation();
    	}

    	/**
     * Mousemove handler for column resizing
     */
    	function onColumnResizeMouseMove(event) {
    		// if not currently resizing a column, ignore the event
    		if (!__resizing) {
    			return;
    		}

    		const { left: wrapperPageX } = wrapper.getBoundingClientRect();
    		const resizeLineLeft = event.pageX - wrapperPageX;

    		const columnLeft = getCellLeft({
    			i: __columnIndexBeingResized,
    			columnWidths,
    			__affixedColumnIndices,
    			__scrollLeft
    		});

    		const resizeLineMinLeft = columnLeft - __scrollLeft + MIN_COLUMN_SIZE;
    		const newColumnWidth = Math.max(resizeLineLeft + __scrollLeft - columnLeft, MIN_COLUMN_SIZE);

    		// thanks to the virtual list, we're able to get away with setting the column's size while the mouse moves
    		$$invalidate(0, columns[__columnIndexBeingResized].width = newColumnWidth, columns);

    		$$invalidate(6, __columnActionLineLeft = Math.max(resizeLineLeft, resizeLineMinLeft));

    		// If mouseup was not fired for some reason, abort the resize
    		if (event.which !== 1) {
    			$$invalidate(5, __resizing = false);
    			$$invalidate(39, __columnIndexBeingResized = null);

    			// delay firing the event until the next frame to guarantee that new values will be available in component.get()
    			setTimeout(
    				() => dispatch("columnWidthUpdated", {
    					idx: __columnIndexBeingResized,
    					width: newColumnWidth
    				}),
    				0
    			);
    		}
    	} // if still resizing and the user does not have the left mouse button depressed,
    	// the mouseup event didn't fire for some reason, so turn off the resize mode

    	/**
     * Mouseup handler for column resizing
     */
    	function onColumnResizeEnd(event) {
    		if (!__resizing) {
    			return;
    		}

    		dispatch("columnWidthUpdated");
    		$$invalidate(5, __resizing = false);
    		$$invalidate(39, __columnIndexBeingResized = null);
    	}

    	/**
     * Sets updated scroll values when the scrollable area is scrolled
     */
    	function onScroll() {
    		// get new scroll values from the scroll area
    		const { scrollTop: newScrollTop, scrollLeft: newScrollLeft } = tableSpace;

    		/*
     * To avoid doing unnecessary re-calculation of computed variables, don't set the scroll
     * properties that haven't changed
     */
    		if (__scrollTop !== newScrollTop) {
    			$$invalidate(40, __scrollTop = newScrollTop);
    		}

    		if (__scrollLeft !== newScrollLeft) {
    			$$invalidate(8, __scrollLeft = newScrollLeft);
    		}

    		$$invalidate(41, __scrolledAllTheWayToTheRight = Math.ceil(tableSpace.scrollWidth - tableSpace.scrollLeft) === tableSpace.clientWidth);
    	}

    	/**
     * Computed Properties
     */
    	/**
     * The 'left' value of the column affix line
     */
    	let columnAffixLineLeft = 0; //TODO setter probably not needed due to reactive statement

    	/**
     * Array of column widths
     */
    	let columnWidths = columns.map(x => x.width || MIN_COLUMN_SIZE); //TODO setter probably not needed due to reactive statement

    	/**
     * The number of rows we have
     */
    	let numRows = rows.length; //TODO setter probably not needed due to reactive statement

    	/**
     * Width of the overall grid space
     */
    	let gridSpaceWidth = 0; //TODO setter probably not needed due to reactive statement

    	/**
     * Height of the overall grid space
     */
    	let gridSpaceHeight = rowHeight * numRows; //TODO setter probably not needed due to reactive statement

    	/**
     * Number of rows to render in the viewport
     */
    	let numRowsInViewport = Math.ceil(__innerOffsetHeight / rowHeight);

    	/**
     * Computes which rows should be visible
     */
    	let visibleRows;

    	/**
     * Helpers
     */
    	const getCellZIndex = function (__affixedColumnIndices, i) {
    		return __affixedColumnIndices.indexOf(i) === -1 ? 1 : 2;
    	};

    	/**
     * Gets the 'top' value for a grid-row
     */
    	const getRowTop = function (i, rowHeight) {
    		return i * rowHeight;
    	};

    	const mousedown_handler = (i, event) => onColumnDragStart(event, i);
    	const mousedown_handler_1 = (i, event) => onColumnResizeStart(event, i);
    	const mousedown_handler_2 = (i, event) => onColumnResizeStart(event, i);
    	const click_handler = row => onRowClick ? onRowClick(row) : null;

    	function div4_binding($$value) {
    		binding_callbacks[$$value ? "unshift" : "push"](() => {
    			$$invalidate(19, tableSpace = $$value);
    		});
    	}

    	function div4_elementresize_handler() {
    		__innerOffsetHeight = this.offsetHeight;
    		$$invalidate(7, __innerOffsetHeight);
    	}

    	function div5_binding($$value) {
    		binding_callbacks[$$value ? "unshift" : "push"](() => {
    			$$invalidate(18, wrapper = $$value);
    		});
    	}

    	$$self.$set = $$props => {
    		if ("rows" in $$props) $$invalidate(36, rows = $$props.rows);
    		if ("columns" in $$props) $$invalidate(0, columns = $$props.columns);
    		if ("rowHeight" in $$props) $$invalidate(9, rowHeight = $$props.rowHeight);
    		if ("allowResizeFromTableCells" in $$props) $$invalidate(10, allowResizeFromTableCells = $$props.allowResizeFromTableCells);
    		if ("allowResizeFromTableHeaders" in $$props) $$invalidate(11, allowResizeFromTableHeaders = $$props.allowResizeFromTableHeaders);
    		if ("allowColumnReordering" in $$props) $$invalidate(12, allowColumnReordering = $$props.allowColumnReordering);
    		if ("allowColumnAffix" in $$props) $$invalidate(13, allowColumnAffix = $$props.allowColumnAffix);
    		if ("onRowClick" in $$props) $$invalidate(14, onRowClick = $$props.onRowClick);
    		if ("__extraRows" in $$props) $$invalidate(42, __extraRows = $$props.__extraRows);
    		if ("__columnHeaderResizeCaptureWidth" in $$props) $$invalidate(15, __columnHeaderResizeCaptureWidth = $$props.__columnHeaderResizeCaptureWidth);
    		if ("__affixedRowIndices" in $$props) $$invalidate(43, __affixedRowIndices = $$props.__affixedRowIndices);
    		if ("__affixedColumnIndices" in $$props) $$invalidate(1, __affixedColumnIndices = $$props.__affixedColumnIndices);
    		if ("__affixingRow" in $$props) $$invalidate(2, __affixingRow = $$props.__affixingRow);
    		if ("__affixingColumn" in $$props) $$invalidate(3, __affixingColumn = $$props.__affixingColumn);
    		if ("__rowActionLineTop" in $$props) $$invalidate(16, __rowActionLineTop = $$props.__rowActionLineTop);
    		if ("__rowAffixLineTop" in $$props) $$invalidate(17, __rowAffixLineTop = $$props.__rowAffixLineTop);
    		if ("__columnAffixLineLeft" in $$props) $$invalidate(44, __columnAffixLineLeft = $$props.__columnAffixLineLeft);
    		if ("__columnDragging" in $$props) $$invalidate(4, __columnDragging = $$props.__columnDragging);
    		if ("__columnIndexBeingDragged" in $$props) $$invalidate(37, __columnIndexBeingDragged = $$props.__columnIndexBeingDragged);
    		if ("__columnDragOffsetX" in $$props) $$invalidate(38, __columnDragOffsetX = $$props.__columnDragOffsetX);
    		if ("__resizing" in $$props) $$invalidate(5, __resizing = $$props.__resizing);
    		if ("__columnIndexBeingResized" in $$props) $$invalidate(39, __columnIndexBeingResized = $$props.__columnIndexBeingResized);
    		if ("__columnActionLineLeft" in $$props) $$invalidate(6, __columnActionLineLeft = $$props.__columnActionLineLeft);
    		if ("__innerOffsetHeight" in $$props) $$invalidate(7, __innerOffsetHeight = $$props.__innerOffsetHeight);
    		if ("__scrollTop" in $$props) $$invalidate(40, __scrollTop = $$props.__scrollTop);
    		if ("__scrollLeft" in $$props) $$invalidate(8, __scrollLeft = $$props.__scrollLeft);
    		if ("__scrolledAllTheWayToTheRight" in $$props) $$invalidate(41, __scrolledAllTheWayToTheRight = $$props.__scrolledAllTheWayToTheRight);
    	};

    	$$self.$$.update = () => {
    		if ($$self.$$.dirty[0] & /*columns*/ 1) {
    			 {
    				// if width was not provided for this column, give it a default value
    				$$invalidate(21, columnWidths = columns.map(x => x.width || MIN_COLUMN_SIZE));
    			}
    		}

    		if ($$self.$$.dirty[0] & /*__affixedColumnIndices, __scrollLeft, columnWidths*/ 2097410) {
    			 {
    				// if no columns are affixed, set the line all the way to the left
    				if (__affixedColumnIndices.length === 0) {
    					$$invalidate(20, columnAffixLineLeft = 0);
    				}

    				let left = __scrollLeft;

    				for (let i = 0; i < __affixedColumnIndices.length; i++) {
    					left += columnWidths[__affixedColumnIndices[i]];
    				}

    				$$invalidate(20, columnAffixLineLeft = left);
    			}
    		}

    		if ($$self.$$.dirty[1] & /*rows*/ 32) {
    			 {
    				$$invalidate(46, numRows = rows.length);
    			}
    		}

    		if ($$self.$$.dirty[0] & /*columnWidths, __resizing*/ 2097184 | $$self.$$.dirty[1] & /*__scrolledAllTheWayToTheRight*/ 1024) {
    			 {
    				let sum = 0;

    				for (let i = 0; i < columnWidths.length; i++) {
    					sum += columnWidths[i];
    				}

    				/**
     * If the table is scrolled all the way to the right, resizing columns could
     * accelerate until the column is the minimum width. Add some extra space on the right
     * to ensure this undesired behavior does not happen. This doesn't seem like a perfect solution
     * but it works for now until I can think of something better.
     */
    				if (__resizing && __scrolledAllTheWayToTheRight) {
    					sum *= 2;
    				}

    				$$invalidate(22, gridSpaceWidth = sum);
    			}
    		}

    		if ($$self.$$.dirty[0] & /*rowHeight*/ 512 | $$self.$$.dirty[1] & /*numRows*/ 32768) {
    			 {
    				$$invalidate(23, gridSpaceHeight = rowHeight * numRows);
    			}
    		}

    		if ($$self.$$.dirty[0] & /*__innerOffsetHeight, rowHeight*/ 640) {
    			 {
    				$$invalidate(47, numRowsInViewport = Math.ceil(__innerOffsetHeight / rowHeight));
    			}
    		}

    		if ($$self.$$.dirty[0] & /*rowHeight*/ 512 | $$self.$$.dirty[1] & /*__scrollTop, __extraRows, numRowsInViewport, rows*/ 68128) {
    			 {
    				const start = Math.max(0, Math.floor(__scrollTop / rowHeight - __extraRows / 2));
    				const end = start + numRowsInViewport + __extraRows;

    				$$invalidate(24, visibleRows = rows.slice(start, end).map((x, i) => {
    					return {
    						i: i + start, // for aria-rowindex
    						data: x, // the row data
    						
    					};
    				}));
    			}
    		}
    	};

    	return [
    		columns,
    		__affixedColumnIndices,
    		__affixingRow,
    		__affixingColumn,
    		__columnDragging,
    		__resizing,
    		__columnActionLineLeft,
    		__innerOffsetHeight,
    		__scrollLeft,
    		rowHeight,
    		allowResizeFromTableCells,
    		allowResizeFromTableHeaders,
    		allowColumnReordering,
    		allowColumnAffix,
    		onRowClick,
    		__columnHeaderResizeCaptureWidth,
    		__rowActionLineTop,
    		__rowAffixLineTop,
    		wrapper,
    		tableSpace,
    		columnAffixLineLeft,
    		columnWidths,
    		gridSpaceWidth,
    		gridSpaceHeight,
    		visibleRows,
    		onWindowKeyDown,
    		onMouseMove,
    		onMouseUp,
    		onCellUpdated,
    		onColumnAffixStart,
    		onRowAffixStart,
    		onColumnDragStart,
    		onColumnResizeStart,
    		onScroll,
    		getCellZIndex,
    		getRowTop,
    		rows,
    		__columnIndexBeingDragged,
    		__columnDragOffsetX,
    		__columnIndexBeingResized,
    		__scrollTop,
    		__scrolledAllTheWayToTheRight,
    		__extraRows,
    		__affixedRowIndices,
    		__columnAffixLineLeft,
    		editHistory,
    		numRows,
    		numRowsInViewport,
    		dispatch,
    		undo,
    		redo,
    		onColumnAffixMouseMove,
    		onColumnAffixEnd,
    		onRowAffixMouseMove,
    		onRowAffixEnd,
    		onColumnDragMouseMove,
    		onColumnDragEnd,
    		onColumnResizeMouseMove,
    		onColumnResizeEnd,
    		mousedown_handler,
    		mousedown_handler_1,
    		mousedown_handler_2,
    		click_handler,
    		div4_binding,
    		div4_elementresize_handler,
    		div5_binding
    	];
    }

    class Src extends SvelteComponent {
    	constructor(options) {
    		super();
    		if (!document_1.getElementById("svelte-b58rik-style")) add_css();

    		init(
    			this,
    			options,
    			instance,
    			create_fragment,
    			safe_not_equal,
    			{
    				rows: 36,
    				columns: 0,
    				rowHeight: 9,
    				allowResizeFromTableCells: 10,
    				allowResizeFromTableHeaders: 11,
    				allowColumnReordering: 12,
    				allowColumnAffix: 13,
    				onRowClick: 14,
    				__extraRows: 42,
    				__columnHeaderResizeCaptureWidth: 15,
    				__affixedRowIndices: 43,
    				__affixedColumnIndices: 1,
    				__affixingRow: 2,
    				__affixingColumn: 3,
    				__rowActionLineTop: 16,
    				__rowAffixLineTop: 17,
    				__columnAffixLineLeft: 44,
    				__columnDragging: 4,
    				__columnIndexBeingDragged: 37,
    				__columnDragOffsetX: 38,
    				__resizing: 5,
    				__columnIndexBeingResized: 39,
    				__columnActionLineLeft: 6,
    				__innerOffsetHeight: 7,
    				__scrollTop: 40,
    				__scrollLeft: 8,
    				__scrolledAllTheWayToTheRight: 41
    			},
    			[-1, -1, -1]
    		);
    	}
    }

    return Src;

})));
